@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-numa.h"
 
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 #include <linux/mempolicy.h>
@@ -30,8 +31,6 @@ static const char option[] = "option --mbind";
     defined(__NR_move_pages) &&         \
     defined(__NR_set_mempolicy) &&	\
     defined(HAVE_LINUX_MEMPOLICY_H)
-
-
 /*
  * stress_check_numa_range()
  * @max_node: maximum NUMA node allowed, 0..N
@@ -62,7 +61,8 @@ int stress_numa_count_mem_nodes(unsigned long *max_node)
 {
 	FILE *fp;
 	unsigned long node_id = 0;
-	char buffer[8192], *str = NULL, *ptr;
+	char buffer[8192], *ptr;
+	const char *str = NULL;
 	long n = 0;
 
 	*max_node = 0;
@@ -117,13 +117,32 @@ int stress_numa_count_mem_nodes(unsigned long *max_node)
 	return n;
 }
 
+/*
+ *  stress_numa_nodes()
+ *	determine the number of NUMA memory nodes,
+ *	always returns at least 1 if no nodes found,
+ *	useful for cache size scaling by node count
+ */
+int stress_numa_nodes(void)
+{
+	unsigned long max_node = 0;
+	static int nodes = -1;	/* used as a cached copy */
+
+	if (nodes == -1) {
+		nodes = stress_numa_count_mem_nodes(&max_node);
+		if (nodes < 1)
+			nodes = 1;
+	}
+	return nodes;
+}
+
 /*
  * stress_parse_node()
  * @str: parse string containing decimal NUMA node number
  *
  * Returns: NUMA node number, or exits the program on invalid number in str
  */
-static unsigned long stress_parse_node(char *const str)
+static unsigned long stress_parse_node(const char *const str)
 {
 	unsigned long val;
 
@@ -209,6 +228,11 @@ int stress_set_mbind(const char *arg)
 }
 
 #else
+int stress_numa_nodes(void)
+{
+	return 1;
+}
+
 int stress_numa_count_mem_nodes(unsigned long *max_node)
 {
 	*max_node = 0;
