@@ -250,7 +250,7 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 
 		ret = write(fd, buffer, count);
 		if (ret >= 0) {
-			n -= ret;
+			n -= STRESS_MINIMUM(ret, n);
 		} else {
 			return -1;
 		}
@@ -267,47 +267,56 @@ static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 int shim_posix_fallocate(int fd, off_t offset, off_t len)
 {
 #if defined(HAVE_POSIX_FALLOCATE) &&	\
-    !defined(__FreeBSD__)
+    !defined(__FreeBSD__) &&		\
+    defined(EINVAL) &&			\
+    defined(EOPNOTSUPP)
 	const off_t chunk_len = (off_t)(1 * MB);
 	static bool emulate = false;
 
-	if (emulate)
-		return shim_emulate_fallocate(fd, offset, len);
+	if (emulate) {
+		errno = 0;
+		if (shim_emulate_fallocate(fd, offset, len) < 0)
+			return errno;
+		return 0;
+	}
 
 	do {
-		const off_t sz = (len > chunk_len) ? chunk_len : len;
 		int ret;
+		const off_t sz = (len > chunk_len) ? chunk_len : len;
 
 		errno = 0;
+		/* posix fallocate returns err in return and not via errno */
 		ret = posix_fallocate(fd, offset, sz);
 		if (ret != 0) {
-			if (errno == EINVAL) {
+			if ((ret == EINVAL) || (ret == EOPNOTSUPP)) {
 				emulate = true;
-				return shim_emulate_fallocate(fd, offset, len);
+				errno = 0;
+				if (shim_emulate_fallocate(fd, offset, len) < 0)
+					return errno;
+				return 0;
 			}
-			errno = ret;
-			return -1;
+			return ret;
 		}
 
 		/*
 		 *  stressor has been signaled to finish, fake
 		 *  an EINTR return
 		 */
-		if (!stress_continue_flag()) {
-			errno = EINTR;
-			return -1;
-		}
+		if (!stress_continue_flag())
+			return EINTR;
 		offset += sz;
 		len -= sz;
 	} while (len > 0);
 
 	return 0;
 #else
-	return shim_emulate_fallocate(fd, offset, len);
+	errno = 0;
+	if (shim_emulate_fallocate(fd, offset, len) < 0)
+		return errno;
+	return 0;
 #endif
 }
 
-
 /*
  * shim_fallocate()
  *	shim wrapper for fallocate system call
@@ -2270,16 +2279,14 @@ long shim_ssetmask(long newmask)
 }
 
 /* use praga to avoid stime already declared warning */
-STRESS_PRAGMA_PUSH
-STRESS_PRAGMA_WARN_OFF
 /*
  *  shim_stime()
  *	wrapper for obsolete SVr4 stime system call
  */
 int shim_stime(const time_t *t)
 {
-#if defined(HAVE_STIME)
-#if defined(__minix__)
+#if defined(HAVE_STIME) &&	\
+    defined(__minix__)
 	/*
 	 *  Minix does not provide the prototype for stime
 	 *  and does not use a const time_t * argument
@@ -2288,11 +2295,9 @@ int shim_stime(const time_t *t)
 	time_t *ut = (time_t *)shim_unconstify_ptr(t);
 
 	return stime(ut);
-#else
-	extern int stime(const time_t *t);
-
+#elif defined(HAVE_STIME) &&	\
+      !NEED_GLIBC(2, 31, 0)
 	return stime(t);
-#endif
 #elif defined(__NR_stime) &&	\
       defined(HAVE_SYSCALL)
 	return (int)syscall(__NR_stime, t);
@@ -2300,7 +2305,6 @@ int shim_stime(const time_t *t)
 	return (int)shim_enosys(0, t);
 #endif
 }
-STRESS_PRAGMA_POP
 
 /*
  *  shim_vhangup()
@@ -2498,17 +2502,17 @@ int shim_unlink(const char *pathname)
  *  shim_unlinkat()
  *	unlinkat, skip operation if --keep-files is enabled
  */
-int shim_unlinkat(int dirfd, const char *pathname, int flags)
+int shim_unlinkat(int dir_fd, const char *pathname, int flags)
 {
 	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
 		return 0;
 #if defined(HAVE_UNLINKAT)
-	return unlinkat(dirfd, pathname, flags);
+	return unlinkat(dir_fd, pathname, flags);
 #elif defined(__NR_unlinkat) &&	\
       defined(HAVE_SYSCALL)
-	return (int)syscall(__NR_unlinkat, dirfd, pathname, flags);
+	return (int)syscall(__NR_unlinkat, dir_fd, pathname, flags);
 #else
-	return (int)shim_enosys(0, dirfd, pathname, flags);
+	return (int)shim_enosys(0, dir_fd, pathname, flags);
 #endif
 }
 
