@@ -44,42 +44,12 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_affinity_delay(const char *opt)
-{
-	uint64_t affinity_delay;
-
-	affinity_delay = stress_get_uint64(opt);
-	stress_check_range("affinity-delay", affinity_delay,
-		0, STRESS_NANOSECOND);
-	return stress_set_setting("affinity-delay", TYPE_ID_UINT64, &affinity_delay);
-}
-
-static int stress_set_affinity_rand(const char *opt)
-{
-	return stress_set_setting_true("affinity-rand", opt);
-}
-
-static int stress_set_affinity_pin(const char *opt)
-{
-	return stress_set_setting_true("affinity-pin", opt);
-}
-
-static int stress_set_affinity_sleep(const char *opt)
-{
-	uint64_t affinity_sleep;
-
-	affinity_sleep = stress_get_uint64(opt);
-	stress_check_range("affinity-sleep", affinity_sleep,
-		0, STRESS_NANOSECOND);
-	return stress_set_setting("affinity-sleep", TYPE_ID_UINT64, &affinity_sleep);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_affinity_delay,	stress_set_affinity_delay },
-	{ OPT_affinity_pin,	stress_set_affinity_pin },
-	{ OPT_affinity_rand,    stress_set_affinity_rand },
-	{ OPT_affinity_sleep,   stress_set_affinity_sleep },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_affinity_delay, "affinity-delay", TYPE_ID_UINT64, 0, STRESS_NANOSECOND, NULL },
+	{ OPT_affinity_pin,   "affinity-pin",	TYPE_ID_BOOL,	0, 1,                 NULL },
+	{ OPT_affinity_rand,  "affinity-rand",  TYPE_ID_BOOL,	0, 1,                 NULL },
+	{ OPT_affinity_sleep, "affinity-sleep", TYPE_ID_UINT64, 0, STRESS_NANOSECOND, NULL },
+	END_OPT,
 };
 
 /*
@@ -245,7 +215,7 @@ static int stress_affinity(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	counter_lock = stress_lock_create();
+	counter_lock = stress_lock_create("counter");
 	if (!counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		(void)stress_s_pids_munmap(s_pids, STRESS_AFFINITY_PROCS);
@@ -262,6 +232,7 @@ static int stress_affinity(stress_args_t *args)
 		(void)stress_s_pids_munmap(s_pids, STRESS_AFFINITY_PROCS);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(info, info_sz, "counters");
 
 	info->affinity_delay = 0;
 	info->affinity_pin = false;
@@ -285,8 +256,9 @@ static int stress_affinity(stress_args_t *args)
 		if (s_pids[i].pid == 0) {
 			s_pids[i].pid = getpid();
 
-			stress_set_proc_state(args->name, STRESS_STATE_RUN);
+			stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 			stress_sync_start_wait_s_pid(&s_pids[i]);
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
 			stress_affinity_child(args, info, false);
 			_exit(EXIT_SUCCESS);
 		} else if (s_pids[i].pid > 0) {
@@ -294,10 +266,10 @@ static int stress_affinity(stress_args_t *args)
 		}
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
 	stress_sync_start_cont_list(s_pids_head);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	stress_affinity_child(args, info, true);
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -320,7 +292,7 @@ stressor_info_t stress_affinity_info = {
 	.stressor = stress_affinity,
 	.class = CLASS_SCHEDULER,
 	.supported = stress_affinity_supported,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 };
@@ -328,7 +300,7 @@ stressor_info_t stress_affinity_info = {
 stressor_info_t stress_affinity_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without sched_getaffinity() or sched_setaffinity()"
