@@ -40,35 +40,11 @@ typedef struct {
 	bool brk_notouch;
 } brk_context_t;
 
-/*
- *  stress_set_bigheap_bytes()
- *     Set maximum allocation amount in bytes
- */
-static int stress_set_brk_bytes(const char *opt)
-{
-	size_t brk_bytes;
-
-	brk_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("brk-bytes", (uint64_t)brk_bytes,
-		MIN_BRK_BYTES, MAX_BRK_BYTES);
-	return stress_set_setting("brk-bytes", TYPE_ID_SIZE_T, &brk_bytes);
-}
-
-static int stress_set_brk_mlock(const char *opt)
-{
-	return stress_set_setting_true("brk-mlock", opt);
-}
-
-static int stress_set_brk_notouch(const char *opt)
-{
-	return stress_set_setting_true("brk-notouch", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_brk_bytes,	stress_set_brk_bytes },
-	{ OPT_brk_mlock,	stress_set_brk_mlock },
-	{ OPT_brk_notouch,	stress_set_brk_notouch },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_brk_bytes,   "brk-bytes",   TYPE_ID_SIZE_T_BYTES, MIN_BRK_BYTES, MAX_BRK_BYTES, NULL },
+	{ OPT_brk_mlock,   "brk-mlock",   TYPE_ID_BOOL,         0,             1,             NULL },
+	{ OPT_brk_notouch, "brk-notouch", TYPE_ID_BOOL,         0,             1,             NULL },
+	END_OPT,
 };
 
 static int stress_brk_supported(const char *name)
@@ -134,8 +110,8 @@ static inline size_t PURE stress_brk_abs(const uint8_t *ptr1, const uint8_t *ptr
 
 static int OPTIMIZE3 stress_brk_child(stress_args_t *args, void *context)
 {
-	uint8_t *start_ptr, *unmap_ptr = NULL;
-	int i = 0;
+	uint8_t *start_ptr, *unmap_ptr = NULL, *brk_failed_ptr = NULL;
+	int i = 0, brk_failed_count = 0;
 	size_t brk_bytes = DEFAULT_BRK_BYTES;
 	const size_t page_size = args->page_size;
 	const brk_context_t *brk_context = (brk_context_t *)context;
@@ -188,6 +164,8 @@ static int OPTIMIZE3 stress_brk_child(stress_args_t *args, void *context)
 				sbrk_exp_count += 1.0;
 
 				ptr += page_size;
+				brk_failed_ptr = NULL;
+				brk_failed_count = 0;
 				if (!unmap_ptr)
 					unmap_ptr = ptr;
 				stress_brk_page_resident(ptr, page_size, brk_touch);
@@ -195,6 +173,16 @@ static int OPTIMIZE3 stress_brk_child(stress_args_t *args, void *context)
 				/* stash a check value */
 				tmp = (uintptr_t *)((uintptr_t)ptr - sizeof(uintptr_t));
 				*tmp = (uintptr_t)tmp;
+			} else {
+				if (brk_failed_ptr == ptr) {
+					brk_failed_count++;
+					if (brk_failed_count > 32) {
+						pr_inf("%s: repeated sbrk page expands ran out of memory, exiting early\n", args->name);
+						break;
+					}
+				}
+				i = 0;
+				brk_failed_ptr = ptr;
 			}
 		} else if (i < 9) {
 			/* brk to same brk position */
@@ -282,8 +270,9 @@ static int stress_brk(stress_args_t *args)
 			args->name);
 	}
 #endif
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, (void *)&brk_context, stress_brk_child, STRESS_OOMABLE_DROP_CAP);
 
@@ -296,7 +285,7 @@ stressor_info_t stress_brk_info = {
 	.stressor = stress_brk,
 	.supported = stress_brk_supported,
 	.class = CLASS_OS | CLASS_VM,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
