@@ -39,19 +39,10 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-/*
- *  stress_set_chdir_dirs()
- *	set number of chdir directories from given option string
- */
-static int stress_set_chdir_dirs(const char *opt)
-{
-	uint32_t chdir_dirs;
-
-	chdir_dirs = stress_get_uint32(opt);
-	stress_check_range("chdir-dirs", (uint64_t)chdir_dirs,
-		MIN_CHDIR_DIRS, MAX_CHDIR_DIRS);
-	return stress_set_setting("chdir-dirs", TYPE_ID_UINT32, &chdir_dirs);
-}
+static const stress_opt_t opts[] = {
+	{ OPT_chdir_dirs, "chdir-dirs", TYPE_ID_UINT32, MIN_CHDIR_DIRS, MAX_CHDIR_DIRS, NULL },
+        END_OPT,
+};
 
 /*
  *  stress_chdir
@@ -150,8 +141,9 @@ static int stress_chdir(stress_args_t *args)
 		goto abort;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		for (i = 0; stress_continue(args) && (i < chdir_dirs); i++) {
@@ -296,15 +288,10 @@ err:
 	return ret;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_chdir_dirs,	stress_set_chdir_dirs },
-	{ 0,			NULL }
-};
-
 stressor_info_t stress_chdir_info = {
 	.stressor = stress_chdir,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
