@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"chown N",	"start N workers thrashing chown file ownership" },
@@ -25,7 +26,7 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-static int OPTIMIZE3 stress_chown_check(const int ret)
+static int PURE OPTIMIZE3 stress_chown_check(const int ret)
 {
 	static const int ignore_errors[] = {
 #if defined(ENOENT)
@@ -161,7 +162,7 @@ restore:
  *  stress_chown
  *	stress chown
  */
-static int stress_chown(const stress_args_t *args)
+static int stress_chown(stress_args_t *args)
 {
 	const pid_t ppid = getppid();
 	int fd = -1, rc = EXIT_FAILURE, retries = 0;
@@ -199,7 +200,7 @@ static int stress_chown(const stress_args_t *args)
 	} else {
 		/* Other instances must try to open the file */
 		for (;;) {
-			if ((fd = open(filename, O_RDWR, S_IRUSR | S_IWUSR)) > - 1)
+			if ((fd = open(filename, O_RDWR)) > - 1)
 				break;
 
 #if defined(__NetBSD__)
@@ -226,34 +227,48 @@ static int stress_chown(const stress_args_t *args)
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+#if defined(HAVE_PATHCONF) &&	\
+    defined(_PC_CHOWN_RESTRICTED)
+	VOID_RET(long, pathconf(filename, _PC_CHOWN_RESTRICTED));
+#endif
+
+	rc = EXIT_SUCCESS;
 	do {
 		int ret;
 
 		ret = do_fchown(fd, bad_fd, cap_chown, uid, gid);
-		if ((ret < 0) && (ret != -EPERM))
+		if ((ret < 0) && (ret != -EPERM)) {
 			pr_fail("%s: fchown failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno),
 				stress_get_fs_type(filename));
+			rc = EXIT_FAILURE;
+			break;
+		}
 
 		ret = do_chown(chown, filename, cap_chown, uid, gid);
 		if ((ret < 0) && (ret != -EPERM)) {
 			pr_fail("%s: chown %s failed, errno=%d (%s)%s\n",
 				args->name, filename, errno, strerror(errno),
 				stress_get_fs_type(filename));
+			rc = EXIT_FAILURE;
+			break;
 		}
 		ret = do_chown(lchown, filename, cap_chown, uid, gid);
 		if ((ret < 0) && (ret != -EPERM)) {
 			pr_fail("%s: lchown %s failed, errno=%d (%s)%s\n",
 				args->name, filename, errno, strerror(errno),
 				stress_get_fs_type(filename));
+			rc = EXIT_FAILURE;
+			break;
 		}
 		(void)shim_fsync(fd);
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
-	rc = EXIT_SUCCESS;
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
