@@ -155,7 +155,7 @@ static int stress_chroot_test1(chroot_shared_data_t *data)
 	 * We check for error, ENOENT can happen on termination
 	 * so ignore this error
 	 */
-	if ((ret1 < 0) && (errno != ENOENT)) {
+	if ((ret1 < 0) && (errno1 != ENOENT)) {
 		pr_fail("%s: chroot(\"%s\"), errno=%d (%s)\n",
 			data->args->name, temppath, errno1, strerror(errno1));
 		return EXIT_FAILURE;
@@ -209,7 +209,7 @@ static int stress_chroot_test3(chroot_shared_data_t *data)
 
 	do_chroot(data, longpath, NULL, &ret1, &ret2, &errno1, &errno2);
 #if defined(__HAIKU__)
-	if ((ret1 >= 0) || (errno1 != EINVAL)) {
+	if ((ret1 >= 0) || ((errno1 != EINVAL) && (errno1 != ENAMETOOLONG))) {
 #else
 	if ((ret1 >= 0) || (errno1 != ENAMETOOLONG)) {
 #endif
@@ -422,6 +422,7 @@ static int stress_chroot(stress_args_t *args)
 			args->name);
 		return EXIT_FAILURE;
 	}
+	stress_set_vma_anon_name(data, sizeof(*data), "metrics");
 	data->args = args;
 	data->rootpath_inode = chroot_inode("/");
 
@@ -442,6 +443,8 @@ static int stress_chroot(stress_args_t *args)
 	(void)close(fd);
 	data->cwd_fd = open(".", O_DIRECTORY | O_RDONLY);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -481,10 +484,11 @@ again:
 			i = 0;
 	} while (stress_continue(args));
 
-	stress_chroot_report_escapes(args, data);
+	if (args->instance == 0)
+		stress_chroot_report_escapes(args, data);
 	rate = (data->metrics.duration > 0.0) ? data->metrics.count / data->metrics.duration : 0.0;
 	stress_metrics_set(args, 0, "chroot calls per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	ret = EXIT_SUCCESS;
 
