@@ -49,6 +49,12 @@ typedef struct dekker {
 
 dekker_t *dekker;
 
+static inline void ALWAYS_INLINE dekker_mfence(void)
+{
+	shim_mfence();
+	stress_asm_mb();
+}
+
 static int stress_dekker_p0(stress_args_t *args)
 {
 	int check0, check1;
@@ -56,15 +62,15 @@ static int stress_dekker_p0(stress_args_t *args)
 
 	t = stress_time_now();
 	dekker->m.wants_to_enter[0] = true;
-	shim_mfence();
+	dekker_mfence();
 	while (LIKELY(dekker->m.wants_to_enter[1])) {
 		if (dekker->m.turn != 0) {
 			dekker->m.wants_to_enter[0] = false;
-			shim_mfence();
+			dekker_mfence();
 			while (dekker->m.turn != 0) {
 			}
 			dekker->m.wants_to_enter[0] = true;
-			shim_mfence();
+			dekker_mfence();
 		}
 	}
 
@@ -72,10 +78,11 @@ static int stress_dekker_p0(stress_args_t *args)
 	check0 = dekker->m.check;
 	dekker->m.check++;
 	check1 = dekker->m.check;
+	dekker_mfence();
 
 	dekker->m.turn = 1;
 	dekker->m.wants_to_enter[0] = false;
-	shim_mfence();
+	dekker_mfence();
 	dekker->p0.duration += stress_time_now() - t;
 	dekker->p0.count += 1.0;
 
@@ -95,15 +102,15 @@ static int stress_dekker_p1(stress_args_t *args)
 	t = stress_time_now();
 
 	dekker->m.wants_to_enter[1] = true;
-	shim_mfence();
+	dekker_mfence();
 	while (LIKELY(dekker->m.wants_to_enter[0])) {
 		if (dekker->m.turn != 1) {
 			dekker->m.wants_to_enter[1] = false;
-			shim_mfence();
+			dekker_mfence();
 			while (dekker->m.turn != 1) {
 			}
 			dekker->m.wants_to_enter[1] = true;
-			shim_mfence();
+			dekker_mfence();
 		}
 	}
 
@@ -111,11 +118,12 @@ static int stress_dekker_p1(stress_args_t *args)
 	check0 = dekker->m.check;
 	dekker->m.check--;
 	check1 = dekker->m.check;
+	dekker_mfence();
 	stress_bogo_inc(args);
 
 	dekker->m.turn = 0;
 	dekker->m.wants_to_enter[1] = false;
-	shim_mfence();
+	dekker_mfence();
 	dekker->p1.duration += stress_time_now() - t;
 	dekker->p1.count += 1.0;
 
@@ -146,8 +154,11 @@ static int stress_dekker(stress_args_t *args)
 			args->name, sz);
 		return EXIT_NO_RESOURCE;
 	}
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
+	stress_set_vma_anon_name(dekker, sz, "dekker-mutex");
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	parent_cpu = stress_get_cpu();
 	pid = fork();
@@ -195,7 +206,7 @@ static int stress_dekker(stress_args_t *args)
 
 stressor_info_t stress_dekker_info = {
 	.stressor = stress_dekker,
-	.class = CLASS_CPU_CACHE,
+	.class = CLASS_CPU_CACHE | CLASS_IPC,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -204,7 +215,7 @@ stressor_info_t stress_dekker_info = {
 
 stressor_info_t stress_dekker_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.class = CLASS_CPU | CLASS_IPC,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without user space memory fencing"
