@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,15 +18,19 @@
  *
  */
 #include "stress-ng.h"
+#include "core-killpid.h"
 
-#define MIN_DIR_DIRS		(64)
-#define MAX_DIR_DIRS		(65536)
-#define DEFAULT_DIR_DIRS	(8192)
+#include <ctype.h>
+#include <sys/file.h>
 
 #if defined(HAVE_LIBGEN_H)
 #include <libgen.h>
 #endif
 
+#define MIN_DIR_DIRS		(64)
+#define MAX_DIR_DIRS		(65536)
+#define DEFAULT_DIR_DIRS	(8192)
+
 static const stress_help_t help[] = {
 	{ NULL,	"dir N",	"start N directory thrashing stressors" },
 	{ NULL,	"dir-dirs N",	"select number of directories to exercise dir on" },
@@ -34,20 +38,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-/*
- *  stress_set_dir_dirs()
- *      set number of dir directories from given option string
- */
-static int stress_set_dir_dirs(const char *opt)
-{
-	uint64_t dir_dirs;
-
-	dir_dirs = stress_get_uint64(opt);
-	stress_check_range("dir-dirs", dir_dirs,
-		MIN_DIR_DIRS, MAX_DIR_DIRS);
-	return stress_set_setting("dir-dirs", TYPE_ID_UINT64, &dir_dirs);
-}
-
 #if defined(__DragonFly__)
 #define d_reclen d_namlen
 #endif
@@ -136,7 +126,7 @@ static inline void stress_dir_mmap(const int dir_fd, const size_t page_size)
  *	read and stat all dentries
  */
 static int stress_dir_read(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *path)
 {
 	DIR *dp;
@@ -146,16 +136,25 @@ static int stress_dir_read(
 	if (!dp)
 		return -1;
 
-	while (keep_stressing(args) && ((de = readdir(dp)) != NULL)) {
+	while (stress_continue(args) && ((de = readdir(dp)) != NULL)) {
 		char filename[PATH_MAX];
 		struct stat statbuf;
+		int fd;
 
+#if !defined(__CYGWIN__)
 		if (de->d_reclen == 0) {
 			pr_fail("%s: read a zero sized directory entry\n", args->name);
 			break;
 		}
+#endif
 		stress_mk_filename(filename, sizeof(filename), path, de->d_name);
-		(void)stat(filename, &statbuf);
+		fd = open(filename, O_RDONLY);
+		if (fd >= 0) {
+			(void)shim_fstat(fd, &statbuf);
+			(void)close(fd);
+		} else {
+			(void)shim_stat(filename, &statbuf);
+		}
 	}
 
 	(void)closedir(dp);
@@ -163,12 +162,65 @@ static int stress_dir_read(
 	return 0;
 }
 
+/*
+ *  stress_dir_rename()
+ *	rename all directories. Try to reproduce fixed in linux
+ * 	commit 9b378f6ad48cfa195ed868db9123c09ee7ec5ea2
+ *	("btrfs: fix infinite directory reads")
+ */
+static int stress_dir_rename(
+	stress_args_t *args,
+	const char *path)
+{
+	DIR *dp;
+	struct dirent *de;
+	char new_filename[PATH_MAX];
+	char tmp[32];
+
+	(void)snprintf(tmp, sizeof(tmp), "rename-%" PRIu32 "-%jd", stress_mwc32(), (intmax_t)getpid());
+	stress_mk_filename(new_filename, sizeof(new_filename), path, tmp);
+
+	dp = opendir(path);
+	if (!dp)
+		return -1;
+
+	while (stress_continue(args) && ((de = readdir(dp)) != NULL)) {
+		char old_filename[PATH_MAX];
+
+#if !defined(__CYGWIN__)
+		if (de->d_reclen == 0) {
+			pr_fail("%s: read a zero sized directory entry\n", args->name);
+			break;
+		}
+#endif
+		if (de->d_name[0] == '.')
+			continue;
+
+		stress_mk_filename(old_filename, sizeof(old_filename), path, de->d_name);
+		if (rename(old_filename, new_filename) < 0) {
+			pr_fail("%s: rename %s to %s failed, errno=%d (%s)\n",
+				args->name, old_filename, new_filename,
+				errno, strerror(errno));
+			break;
+		}
+		if (rename(new_filename, old_filename) < 0) {
+			pr_fail("%s: rename %s to %s failed, errno=%d (%s)\n",
+				args->name, new_filename, old_filename,
+				errno, strerror(errno));
+			break;
+		}
+	}
+	(void)closedir(dp);
+
+	return 0;
+}
+
 /*
  *  stress_dir_tidy()
  *	remove all dentries
  */
 static void stress_dir_tidy(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const uint64_t n)
 {
 	uint64_t i;
@@ -198,7 +250,7 @@ static int stress_mkdir(const int dir_fd, const char *path, const int mode)
 	if ((dir_fd >= 0) && stress_mwc1()) {
 		char tmp[PATH_MAX], *filename;
 
-		(void)shim_strlcpy(tmp, path, sizeof(tmp));
+		(void)shim_strscpy(tmp, path, sizeof(tmp));
 		filename = basename(tmp);
 
 		ret = mkdirat(dir_fd, filename, (shim_mode_t)mode);
@@ -223,7 +275,7 @@ static void stress_invalid_mkdir(const char *path)
 	char filename[PATH_MAX + 16];
 	size_t len;
 
-	(void)shim_strlcpy(filename, path, sizeof(filename));
+	(void)shim_strscpy(filename, path, sizeof(filename));
 	(void)shim_strlcat(filename, "/", sizeof(filename));
 	len = strlen(filename);
 	(void)stress_rndstr(filename + len, sizeof(filename) - len);
@@ -253,7 +305,7 @@ static void stress_invalid_rmdir(const char *path)
 {
 	char filename[PATH_MAX + 16];
 
-	(void)shim_strlcpy(filename, path, sizeof(filename));
+	(void)shim_strscpy(filename, path, sizeof(filename));
 	/* remove . - exercise EINVAL error */
 	(void)shim_strlcat(filename, "/.", sizeof(filename));
 	VOID_RET(int, shim_rmdir(filename));
@@ -273,7 +325,7 @@ static void stress_invalid_rmdir(const char *path)
  *	the directory
  */
 static void stress_dir_read_concurrent(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *pathname)
 {
 	shim_nice(1);
@@ -283,14 +335,113 @@ static void stress_dir_read_concurrent(
 	do {
 		if (stress_dir_read(args, pathname) < 0)
 			return;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
+}
+
+/*
+ *  stress_dir_touch()
+ *	create a small file
+ */
+static int stress_dir_touch(
+	stress_args_t *args,
+	const char *filename)
+{
+	int fd;
+
+	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
+	if (fd < 0) {
+		pr_fail("%s: cannot create file %s, errno=%d (%s)\n",
+			args->name, filename, errno, strerror(errno));
+		return -1;
+	}
+	if (write(fd, "data", 4) < 0) {
+		pr_inf("%s: failed to write to file %s, errno=%d (%s)\n",
+			args->name, filename, errno, strerror(errno));
+		(void)close(fd);
+		return -1;
+	}
+	(void)close(fd);
+	return 0;
+}
+
+/*
+ *  stress_dir_readdir()
+ *	exercise populating a new directory and checking to see if
+ *	rewinddir and readdir pick up all the new entries.
+ */
+static int stress_dir_readdir(
+	stress_args_t *args,
+	const char *pathname)
+{
+	DIR *dir;
+	char dirpath[PATH_MAX + 64];
+	char filename[PATH_MAX + 70];
+	int rc = 0, i, got_mask, all_mask;
+	struct dirent *entry;
+
+	(void)snprintf(dirpath, sizeof(dirpath), "%s/test-%jd-%" PRIu32, pathname,
+		(intmax_t)getpid(), stress_mwc32());
+	if (mkdir(dirpath, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
+		pr_fail("%s: cannot mkdir %s, errno=%d (%s)\n",
+			args->name, dirpath, errno, strerror(errno));
+		rc = -1;
+		goto err_rmdir;
+	}
+
+	/*
+	 *  Check if readdir + rewinddir will pick up new files
+	 */
+	dir = opendir(dirpath);
+	if (!dir) {
+		pr_fail("%s: cannot opendir %s, errno=%d (%s)\n",
+			args->name, dirpath, errno, strerror(errno));
+		rc = -1;
+		goto err_rmdir;
+	}
+	all_mask = 0;
+	for (i = 0; i < 10; i++) {
+		all_mask |= (1U << i);
+		(void)snprintf(filename, sizeof(filename), "%s/%d", dirpath, i);
+		if (stress_dir_touch(args, filename) < 0) {
+			(void)closedir(dir);
+			rc = -1;
+			goto err_rm_files;
+		}
+	}
+
+	rewinddir(dir);
+	got_mask = 0;
+	while ((entry = readdir(dir))) {
+		if (isdigit((int)entry->d_name[0])) {
+			const int d = atoi(entry->d_name);
+
+			got_mask |= (1U << d);
+		}
+	}
+	(void)closedir(dir);
+
+	if (got_mask != all_mask) {
+		pr_fail("%s: rewinddir and readdir did not find all the files in directory %s\n",
+			args->name, dirpath);
+		rc = -1;
+	}
+
+err_rm_files:
+	for (i = 0; i < 10; i++) {
+		(void)snprintf(filename, sizeof(filename), "%s/%d", dirpath, i);
+		(void)unlink(filename);
+	}
+err_rmdir:
+	(void)rmdir(dirpath);
+
+	return rc;
 }
 
 /*
  *  stress_dir
  *	stress directory mkdir and rmdir
  */
-static int stress_dir(const stress_args_t *args)
+static int stress_dir(stress_args_t *args)
 {
 	int ret;
 	uint64_t dir_dirs = DEFAULT_DIR_DIRS;
@@ -299,7 +450,8 @@ static int stress_dir(const stress_args_t *args)
 	const int bad_fd = stress_get_bad_fd();
 	pid_t pid;
 
-	stress_temp_dir(pathname, sizeof(pathname), args->name, args->pid, args->instance);
+	stress_temp_dir(pathname, sizeof(pathname), args->name,
+		args->pid, args->instance);
 	(void)stress_get_setting("dir-dirs", &dir_dirs);
 
 	ret = stress_temp_dir_mk_args(args);
@@ -309,6 +461,8 @@ static int stress_dir(const stress_args_t *args)
 #if defined(O_DIRECTORY)
 	dir_fd = open(pathname, O_DIRECTORY | O_RDONLY);
 #endif
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	pid = fork();
@@ -324,7 +478,7 @@ static int stress_dir(const stress_args_t *args)
 		stress_dir_flock(dir_fd);
 		stress_dir_truncate(pathname, dir_fd);
 
-		for (i = 0; keep_stressing(args) && (i < n); i++) {
+		for (i = 0; stress_continue(args) && (i < n); i++) {
 			char path[PATH_MAX];
 			const uint64_t gray_code = (i >> 1) ^ i;
 
@@ -337,29 +491,41 @@ static int stress_dir(const stress_args_t *args)
 				    (errno != EMLINK)) {
 					pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
 						args->name, path, errno, strerror(errno));
+					ret = EXIT_FAILURE;
 					break;
 				}
 			}
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
 		stress_invalid_mkdir(pathname);
 		stress_invalid_rmdir(pathname);
 		stress_invalid_mkdirat(bad_fd);
 
-		if (!keep_stressing(args)) {
+		if (!stress_continue(args)) {
+			stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 			stress_dir_tidy(args, i);
 			break;
 		}
 		stress_dir_read(args, pathname);
-		stress_dir_tidy(args, i);
-
-		if (!keep_stressing(args))
+		if (!stress_continue(args)) {
+			stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+			stress_dir_tidy(args, i);
 			break;
+		}
+		stress_dir_rename(args, pathname);
+		if (!stress_continue(args)) {
+			stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+			stress_dir_tidy(args, i);
+			break;
+		}
+		if (stress_dir_readdir(args, pathname) < 0)
+			ret = EXIT_FAILURE;
+		stress_dir_tidy(args, i);
 		stress_dir_sync(dir_fd);
 		(void)sync();
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	/* exercise invalid path */
 	{
@@ -375,27 +541,22 @@ static int stress_dir(const stress_args_t *args)
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (pid >= 0) {
-		int status;
-
-		VOID_RET(int, kill(pid, SIGKILL));
-		VOID_RET(int, waitpid(pid, &status, 0));
-	}
-
+	if (pid >= 0)
+		(void)stress_kill_pid_wait(pid, NULL);
 	(void)stress_temp_dir_rm_args(args);
 
 	return ret;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_dir_dirs,	stress_set_dir_dirs },
-	{ 0,		NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_dir_dirs, "dir-dirs", TYPE_ID_UINT64, MIN_DIR_DIRS, MAX_DIR_DIRS, NULL },
+	END_OPT,
 };
 
 stressor_info_t stress_dir_info = {
 	.stressor = stress_dir,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
