@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
 #include "core-builtin.h"
 
 #if defined(HAVE_LINUX_FUTEX_H)
@@ -90,19 +91,21 @@ static int stress_futex_wait(uint32_t *futex, const int val, const long nsec)
  * 	efficiently use futex, but to stress the futex system call
  *	by rapidly calling it on wait and wakes
  */
-static int stress_futex(const stress_args_t *args)
+static int stress_futex(stress_args_t *args)
 {
 	uint64_t *timeout = &g_shared->futex.timeout[args->instance];
 	uint32_t *futex = &g_shared->futex.futex[args->instance];
 	pid_t pid;
-	int parent_cpu;
+	int parent_cpu, rc = EXIT_SUCCESS;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		if (!stress_continue(args))
 			goto finish;
@@ -116,8 +119,8 @@ again:
 			int ret;
 
 			/*
-			 * Break early in case wake gets stuck
-			 * (which it shouldn't)
+			 *  Break early in case wake gets stuck
+			 *  (which it shouldn't)
 			 */
 			if (!stress_continue_flag())
 				break;
@@ -126,6 +129,7 @@ again:
 				if (ret < 0) {
 					pr_fail("%s: futex_wake failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 		} while (stress_continue(args));
@@ -169,28 +173,30 @@ again:
 					if (errno != EINTR) {
 						pr_fail("%s: futex_wait failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
 					}
 				}
 				stress_bogo_inc(args);
 			}
 		} while (stress_continue(args));
+		_exit(rc);
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_futex_info = {
 	.stressor = stress_futex,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
 stressor_info_t stress_futex_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without linux/futex.h or futex() system call"
