@@ -31,14 +31,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		  NULL }
 };
 
-static int stress_set_hrtimers_adjust(const char *opt)
-{
-        return stress_set_setting_true("hrtimers-adjust", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_hrtimers_adjust,	stress_set_hrtimers_adjust },
-	{ 0,			NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_hrtimers_adjust, "hrtimers-adjust", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_LIB_RT) &&		\
@@ -71,7 +66,7 @@ static void stress_hrtimers_set(struct itimerspec *timer)
  *  stress_hrtimers_stress_continue(args)
  *      returns true if we can keep on running a stressor
  */
-static bool HOT OPTIMIZE3 stress_hrtimers_stress_continue(void)
+static bool OPTIMIZE3 stress_hrtimers_stress_continue(void)
 {
 	return (LIKELY(stress_continue_flag()) &&
 		LIKELY(!max_ops || ((timer_counter) < max_ops)));
@@ -134,14 +129,17 @@ static int stress_hrtimer_process(stress_args_t *args)
 	struct sigevent sev;
 	struct itimerspec timer;
 	sigset_t mask;
+	int sched;
 
 	time_end = args->time_end;
 
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
-
-	VOID_RET(int, stress_set_sched(getpid(), SCHED_RR, UNDEFINED, true));
+	/* If sched is not set, use SCHED_RR as default */
+	if (!stress_get_setting("sched", &sched)) {
+		VOID_RET(int, stress_set_sched(getpid(), SCHED_RR, UNDEFINED, true));
+	}
 
 	(void)shim_memset(&action, 0, sizeof action);
 	action.sa_handler = stress_hrtimers_handler;
@@ -188,19 +186,27 @@ static int stress_hrtimer_process(stress_args_t *args)
 
 static int stress_hrtimers(stress_args_t *args)
 {
-	pid_t pids[PROCS_MAX];
+	stress_pid_t *s_pids, *s_pids_head = NULL;
 	size_t i;
-        bool hrtimers_adjust = false;
+	bool hrtimers_adjust = false;
 	double start_time = -1.0, end_time;
 	sigset_t mask;
+	int rc = EXIT_SUCCESS;
 
 	if (stress_sigchld_set_handler(args) < 0)
 		return EXIT_NO_RESOURCE;
 
-	lock = stress_lock_create();
+	s_pids = stress_s_pids_mmap(PROCS_MAX);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, PROCS_MAX);
+		return EXIT_NO_RESOURCE;
+	}
+
+	lock = stress_lock_create("counter");
 	if (!lock) {
 		pr_inf("%s: cannot create lock, skipping stressor\n", args->name);
-		return EXIT_NO_RESOURCE;
+		rc = EXIT_NO_RESOURCE;
+		goto tidy_s_pids;
 	}
 
         (void)stress_get_setting("hrtimers-adjust", &hrtimers_adjust);
@@ -208,26 +214,35 @@ static int stress_hrtimers(stress_args_t *args)
 	ns_delay = hrtimers_adjust ? 10000 : -1;
 	max_ops = args->max_ops / PROCS_MAX;
 
-	(void)shim_memset(pids, 0, sizeof(pids));
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
 	for (i = 0; i < PROCS_MAX; i++) {
+		stress_sync_start_init(&s_pids[i]);
+
 		if (!stress_continue(args))
 			goto reap;
 
-		pids[i] = fork();
-		if (pids[i] == 0) {
+		s_pids[i].pid = fork();
+		if (s_pids[i].pid == 0) {
 			/* Child */
+			s_pids[i].pid = getpid();
+			stress_sync_start_wait_s_pid(&s_pids[i]);
+
 			stress_parent_died_alarm();
 			stress_set_oom_adjustment(args, true);
 			(void)sched_settings_apply(true);
 			stress_hrtimer_process(args);
 			_exit(EXIT_SUCCESS);
+		} else if (s_pids[i].pid > 0) {
+			stress_sync_start_s_pid_list_add(&s_pids_head, &s_pids[i]);
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_sync_start_cont_list(s_pids_head);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	(void)sigemptyset(&mask);
-        (void)sigaddset(&mask, SIGRTMIN);
+	(void)sigaddset(&mask, SIGRTMIN);
 	(void)sigprocmask(SIG_BLOCK, &mask, NULL);
 
 	start_time = stress_time_now();
@@ -238,7 +253,7 @@ static int stress_hrtimers(stress_args_t *args)
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	stress_kill_and_wait_many(args, pids, PROCS_MAX, SIGALRM, true);
+	stress_kill_and_wait_many(args, s_pids, PROCS_MAX, SIGALRM, true);
 	end_time = stress_time_now();
 
 	if (start_time >= 0.0) {
@@ -249,18 +264,20 @@ reap:
 
 			pr_dbg("%s: hrtimer signals at %.3f MHz\n", args->name, rate / 1000000.0);
 			stress_metrics_set(args, 0, "hrtimer signals per sec",
-				rate, STRESS_HARMONIC_MEAN);
+				rate, STRESS_METRIC_HARMONIC_MEAN);
 		}
 	}
 	stress_lock_destroy(lock);
+tidy_s_pids:
+	(void)stress_s_pids_munmap(s_pids, PROCS_MAX);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_hrtimers_info = {
 	.stressor = stress_hrtimers,
 	.class = CLASS_SCHEDULER,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -268,7 +285,7 @@ stressor_info_t stress_hrtimers_info = {
 stressor_info_t stress_hrtimers_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without librt or hrtimer support"
