@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King
+ * Copyright (C) 2021-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,7 @@ static const stress_help_t help[] = {
      defined(HAVE_MPROTECT)
 
 static int icache_madvise_nohugepage(
-	const stress_args_t *args,
+	stress_args_t *args,
 	void *addr,
 	size_t size)
 {
@@ -63,7 +63,7 @@ static int icache_madvise_nohugepage(
 }
 
 static int NOINLINE icache_mprotect(
-	const stress_args_t *args,
+	stress_args_t *args,
 	void *addr,
 	size_t size,
 	int prot)
@@ -78,7 +78,7 @@ static int NOINLINE icache_mprotect(
 	return ret;
 }
 
-static int stress_icache_func(const stress_args_t *args, void *page, const size_t page_size)
+static int stress_icache_func(stress_args_t *args, void *page, const size_t page_size)
 {
 	volatile uint32_t *vaddr = (volatile uint32_t *)page;
 	stress_ret_func_t icache_func = (stress_ret_func_t)page;
@@ -119,7 +119,7 @@ static int stress_icache_func(const stress_args_t *args, void *page, const size_
 			 * as the default code assumes smart x86 style
 			 * I$ behaviour.
 			 */
-			shim_flush_icache((char *)page, (char *)page+ 64);
+			shim_flush_icache((char *)page, (char *)page + 64);
 			*vaddr = val;
 			shim_flush_icache((char *)page, (char *)page + 64);
 			/*
@@ -145,13 +145,14 @@ static int stress_icache_func(const stress_args_t *args, void *page, const size_
  *	I-cache load misses can be observed using:
  *      perf stat -e L1-icache-load-misses stress-ng --icache 0 -t 1
  */
-static int stress_icache(const stress_args_t *args)
+static int stress_icache(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	void *page;
 	int ret;
 
-	page = mmap(NULL, page_size, PROT_READ | PROT_WRITE | PROT_EXEC,
+	page = stress_mmap_populate(NULL, page_size,
+			PROT_READ | PROT_WRITE | PROT_EXEC,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (page == MAP_FAILED) {
 		pr_inf_skip("%s: could not mmap %zd sized page, skipping stressor\n",
@@ -160,6 +161,8 @@ static int stress_icache(const stress_args_t *args)
 	}
 	(void)shim_memcpy(page, &stress_ret_opcode.opcodes, stress_ret_opcode.len);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 	ret = stress_icache_func(args, page, page_size);
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
