@@ -55,6 +55,8 @@ static int stress_kill(stress_args_t *args)
 		_exit(0);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -83,7 +85,7 @@ static int stress_kill(stress_args_t *args)
 		t = stress_time_now();
 		ret = kill(args->pid, SIGUSR1);
 		if (ret == 0) {
-			int saved_errno = errno;
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
@@ -97,7 +99,7 @@ static int stress_kill(stress_args_t *args)
 		t = stress_time_now();
 		ret = kill(args->pid, 0);
 		if (ret == 0) {
-			int saved_errno = errno;
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
@@ -115,7 +117,7 @@ static int stress_kill(stress_args_t *args)
 		t = stress_time_now();
 		ret = kill(-1, 0);
 		if (ret == 0) {
-			int saved_errno = errno;
+			const int saved_errno = errno;
 
 			duration += stress_time_now() - t;
 			count += 1.0;
@@ -169,7 +171,7 @@ static int stress_kill(stress_args_t *args)
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "kill calls per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	return EXIT_SUCCESS;
 }
