@@ -43,20 +43,9 @@ static const stress_help_t symlink_help[] = {
 
 #define MOUNTS_MAX	(128)
 
-static int stress_set_link_sync(const char *opt)
-{
-	return stress_set_setting_true("link-sync", opt);
-}
-
-static int stress_set_symlink_sync(const char *opt)
-{
-	return stress_set_setting_true("symlink-sync", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_link_sync,	stress_set_link_sync },
-	{ OPT_symlink_sync,	stress_set_symlink_sync },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_link_sync,    "link-sync",    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_symlink_sync, "symlink-sync", TYPE_ID_BOOL, 0, 1, NULL },
 };
 
 /*
@@ -105,6 +94,7 @@ static int stress_link_generic(
 	bool symlink_func = (linkfunc == symlink);
 	char *mnts[MOUNTS_MAX];
 	double t_start, duration, rate, link_count = 0.0;
+	char dir_path[PATH_MAX];
 
 	(void)shim_memset(tmp_newpath, 0, sizeof(tmp_newpath));
 	(void)snprintf(tmp_newpath, sizeof(tmp_newpath),
@@ -114,14 +104,13 @@ static int stress_link_generic(
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
 		return stress_exit_status(-ret);
-#if defined(O_DIRECTORY)
-	if (do_sync) {
-		char dir_path[PATH_MAX];
 
-		stress_temp_dir(dir_path, sizeof(dir_path), args->name, args->pid, args->instance);
+	stress_temp_dir(dir_path, sizeof(dir_path), args->name, args->pid, args->instance);
+#if defined(O_DIRECTORY)
+	if (do_sync)
 		temp_dir_fd = open(dir_path, O_RDONLY | O_DIRECTORY);
-	}
 #else
+	(void)dir_path;
 	(void)do_sync;
 #endif
 
@@ -144,8 +133,9 @@ static int stress_link_generic(
 	mounts_max = stress_mount_get(mnts, MOUNTS_MAX);
 	oldpathlen = strlen(oldpath);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = EXIT_SUCCESS;
 	t_start = stress_time_now();
@@ -258,6 +248,21 @@ static int stress_link_generic(
 #endif
 		}
 
+#if defined(HAVE_PATHCONF)
+#if defined(_PC_LINK_MAX)
+		/* exercise pathconf maximum file link count */
+		VOID_RET(long, pathconf(oldpath, _PC_LINK_MAX));
+#endif
+#if defined(_PC_SYMLINK_MAX)
+		/* exercise pathconf maximum file symlink count */
+		VOID_RET(long, pathconf(oldpath, _PC_SYMLINK_MAX));
+#endif
+#if defined(_PC_2_SYMLINKS)
+		/* exercise pathconf maximum file symlink count */
+		VOID_RET(long, pathconf(dir_path, _PC_2_SYMLINKS));
+#endif
+#endif
+
 		/* exercise invalid newpath size, EINVAL */
 		VOID_RET(ssize_t, readlink(oldpath, testpath, 0));
 
@@ -344,7 +349,7 @@ stressor_info_t stress_link_info = {
 	.stressor = stress_link,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = hardlink_help
 };
 #else
@@ -352,7 +357,7 @@ stressor_info_t stress_link_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = hardlink_help,
 	.unimplemented_reason = "unsupported on Haiku"
 };
@@ -362,6 +367,6 @@ stressor_info_t stress_symlink_info = {
 	.stressor = stress_symlink,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = symlink_help,
 };
