@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,8 +19,11 @@
 #include "stress-ng.h"
 #include "core-capabilities.h"
 #include "core-cpu-cache.h"
+#include "core-numa.h"
 #include "core-target-clones.h"
 
+#include <sched.h>
+
 static const stress_help_t help[] = {
 	{ NULL,	"llc-affinity N",	"start N workers exercising low level cache over all CPUs" },
 	{ NULL,	"llc-affinity-mlock",	"attempt to mlock pages into memory" },
@@ -28,14 +31,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_llc_affinity_mlock(const char *opt)
-{
-	return stress_set_setting_true("llc-affinity-mlock", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_llc_affinity_mlock,	stress_set_llc_affinity_mlock },
-	{ 0,				NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_llc_affinity_mlock, "llc-affinity-mlock", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SCHED_SETAFFINITY)
@@ -152,7 +150,7 @@ static void TARGET_CLONES OPTIMIZE3 stress_llc_read_cache_line_n(
  *  stress_llc_affinity()
  *	stress the Lower Level Cache (LLC) while changing CPU affinity
  */
-static int stress_llc_affinity(const stress_args_t *args)
+static int stress_llc_affinity(stress_args_t *args)
 {
 	const int32_t max_cpus = stress_get_processors_configured();
 	const size_t page_size = args->page_size;
@@ -163,6 +161,9 @@ static int stress_llc_affinity(const stress_args_t *args)
 	double write_duration, read_duration, rate, writes, reads, t_start, duration;
 	cache_line_func_t write_func, read_func;
 	bool llc_affinity_mlock = false;
+	const int numa_nodes = stress_numa_nodes();
+
+	stress_catch_sigill();
 
 	(void)stress_get_setting("llc-affinity-mlock", &llc_affinity_mlock);
 
@@ -172,9 +173,15 @@ static int stress_llc_affinity(const stress_args_t *args)
 			args->name);
 		return EXIT_NO_RESOURCE;
 	}
-	if (!args->instance) {
-		pr_inf("%s: system has %zu KB LLC cache\n",
-			args->name, llc_size / 1024);
+        llc_size *= numa_nodes;
+        if (!args->instance) {
+		if (numa_nodes > 1)  {
+			pr_inf("%s: scaling LLC cache size by number of numa nodes %d to %zdK\n",
+				args->name, numa_nodes, llc_size / 1024);
+		} else  {
+			pr_inf("%s: using LLC cache size of %zuK\n",
+				args->name, llc_size / 1024);
+		}
 	}
 
 	mmap_sz = STRESS_MAXIMUM(max_cpus * page_size, llc_size);
@@ -182,16 +189,19 @@ static int stress_llc_affinity(const stress_args_t *args)
 	/*
 	 *  Allocate a LLC sized buffer to exercise
 	 */
-	buf = (uint64_t *)mmap(NULL, mmap_sz, PROT_READ | PROT_WRITE,
-		MAP_ANONYMOUS | MAP_PRIVATE, 0, 0);
+	buf = (uint64_t *)stress_mmap_populate(NULL, mmap_sz, PROT_READ | PROT_WRITE,
+		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buf == MAP_FAILED) {
 		pr_fail("%s: mmap'd region of %zu bytes failed\n", args->name, mmap_sz);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buf, mmap_sz, "llc-buffer");
 	if (llc_affinity_mlock)
 		(void)shim_mlock(buf, mmap_sz);
 	buf_end = (uint64_t *)((uintptr_t)buf + mmap_sz);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	writes = 0.0;
@@ -234,12 +244,15 @@ static int stress_llc_affinity(const stress_args_t *args)
 
 	writes /= (double)MB;
 	rate = write_duration > 0.0 ? (double)writes / write_duration : 0.0;
-	stress_metrics_set(args, 0, "MB pec sec memory write rate", rate);
+	stress_metrics_set(args, 0, "MB pec sec memory write rate",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 	reads /= (double)MB;
 	rate = read_duration > 0.0 ? (double)reads / read_duration : 0.0;
-	stress_metrics_set(args, 1, "MB per sec memory read rate", rate);
+	stress_metrics_set(args, 1, "MB per sec memory read rate",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 	rate = duration > 0.0 ? (double)affinity_changes / duration : 0.0;
-	stress_metrics_set(args, 2, "CPU affinity changes per sec", rate);
+	stress_metrics_set(args, 2, "CPU affinity changes per sec",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)munmap((void *)buf, mmap_sz);
@@ -250,7 +263,7 @@ static int stress_llc_affinity(const stress_args_t *args)
 stressor_info_t stress_llc_affinity_info = {
 	.stressor = stress_llc_affinity,
 	.class = CLASS_CPU_CACHE,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -260,7 +273,7 @@ stressor_info_t stress_llc_affinity_info = {
 stressor_info_t stress_llc_affinity_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sched_setaffinity() support"
