@@ -27,14 +27,9 @@ static const stress_help_t help[] = {
 	{ NULL, NULL,			NULL }
 };
 
-static int stress_set_lockbus_nosplit(const char *opt)
-{
-	return stress_set_setting_true("lockbus-nosplit", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_lockbus_nosplit,	stress_set_lockbus_nosplit },
-	{ 0,			NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_lockbus_nosplit, "lockbus-nosplit", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if (((defined(HAVE_COMPILER_GCC_OR_MUSL) ||		\
@@ -169,6 +164,7 @@ static int stress_lockbus(stress_args_t *args)
 		pr_err("%s: mmap failed\n", args->name);
 		return rc;
 	}
+	stress_set_vma_anon_name(buffer, BUFFER_SIZE, "lockbus-data");
 
 #if defined(STRESS_ARCH_X86)
 	/* Split lock on a page boundary */
@@ -182,6 +178,9 @@ static int stress_lockbus(stress_args_t *args)
 	if (sigsetjmp(jmp_env, 1) && !stress_continue(args))
 		goto done;
 #endif
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	duration = 0;
@@ -245,7 +244,7 @@ done:
 
 	rate = (count > 0.0) ? duration / count : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per memory lock operation",
-		rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 
 	(void)munmap((void *)buffer, BUFFER_SIZE);
 
@@ -255,14 +254,14 @@ done:
 stressor_info_t stress_lockbus_info = {
 	.stressor = stress_lockbus,
 	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
 #else
 stressor_info_t stress_lockbus_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU_CACHE | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without gcc __atomic* lock builtins"
 };
