@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"lockf N",	  "start N workers locking a single file via lockf" },
@@ -47,14 +50,9 @@ typedef struct {
 static stress_lockf_info_list_t lockf_infos;
 #endif
 
-static int stress_lockf_set_nonblock(const char *opt)
-{
-	return stress_set_setting_true("lockf-nonblock", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_lockf_nonblock,   stress_lockf_set_nonblock },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_lockf_nonblock, "lockf-nonblock", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_LOCKF)
@@ -138,7 +136,7 @@ static void stress_lockf_info_free(void)
  *  stress_lockf_unlock()
  *	pop oldest lock record off list and unlock it
  */
-static int stress_lockf_unlock(const stress_args_t *args, const int fd)
+static int stress_lockf_unlock(stress_args_t *args, const int fd)
 {
 	/* Pop one off list */
 	if (!lockf_infos.head)
@@ -164,7 +162,7 @@ static int stress_lockf_unlock(const stress_args_t *args, const int fd)
  *	hammer lock/unlock to create some file lock contention
  */
 static int stress_lockf_contention(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const int bad_fd)
 {
@@ -186,14 +184,14 @@ static int stress_lockf_contention(
 				return -1;
 
 		offset = (off_t)stress_mwc64modn(LOCK_FILE_SIZE - LOCK_SIZE);
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		if (lseek(fd, offset, SEEK_SET) < 0) {
 			pr_err("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return -1;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 		rc = lockf(fd, lockf_cmd, LOCK_SIZE);
 		if (rc < 0) {
@@ -201,8 +199,6 @@ static int stress_lockf_contention(
 				return -1;
 			continue;
 		}
-		if (!keep_stressing_flag())
-			break;
 
 		/* Locked OK, add to lock list */
 		lockf_info = stress_lockf_info_new();
@@ -215,18 +211,18 @@ static int stress_lockf_contention(
 		 *  Occasionally exercise lock on a bad fd, ignore error
 		 */
 		if (counter++ >= 65536) {
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			VOID_RET(int, lockf(bad_fd, lockf_cmd, LOCK_SIZE));
 			counter = 0;
 
-			if (!keep_stressing_flag())
+			if (!stress_continue_flag())
 				break;
 			/* Exercise F_TEST, ignore result */
 			VOID_RET(int, lockf(fd, F_TEST, LOCK_SIZE));
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	return 0;
 }
@@ -235,9 +231,9 @@ static int stress_lockf_contention(
  *  stress_lockf
  *	stress file locking via lockf()
  */
-static int stress_lockf(const stress_args_t *args)
+static int stress_lockf(stress_args_t *args)
 {
-	int fd, ret = EXIT_FAILURE;
+	int fd, ret = EXIT_FAILURE, parent_cpu;
 	const int bad_fd = stress_get_bad_fd();
 	pid_t cpid = -1;
 	char filename[PATH_MAX];
@@ -246,7 +242,7 @@ static int stress_lockf(const stress_args_t *args)
 	off_t offset;
 	ssize_t rc;
 
-	(void)memset(buffer, 0, sizeof(buffer));
+	(void)shim_memset(buffer, 0, sizeof(buffer));
 
 	/*
 	 *  There will be a race to create the directory
@@ -285,7 +281,7 @@ static int stress_lockf(const stress_args_t *args)
 	}
 	for (offset = 0; offset < LOCK_FILE_SIZE; offset += sizeof(buffer)) {
 redo:
-		if (!keep_stressing_flag()) {
+		if (!stress_continue_flag()) {
 			ret = EXIT_SUCCESS;
 			goto tidy;
 		}
@@ -300,19 +296,23 @@ redo:
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	cpid = fork();
 	if (cpid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (!stress_continue(args))
 			goto tidy;
 		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
 	if (cpid == 0) {
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -327,12 +327,8 @@ again:
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (cpid > 0) {
-		int status;
-
-		(void)kill(cpid, SIGALRM);
-		(void)shim_waitpid(cpid, &status, 0);
-	}
+	if (cpid > 1)
+		stress_kill_and_wait(args, cpid, SIGALRM, true);
 	stress_lockf_info_free();
 
 	(void)close(fd);
@@ -345,7 +341,7 @@ tidy:
 stressor_info_t stress_lockf_info = {
 	.stressor = stress_lockf,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -353,7 +349,8 @@ stressor_info_t stress_lockf_info = {
 stressor_info_t stress_lockf_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without lockf() system call"
 };
