@@ -74,8 +74,9 @@ typedef struct {
 #endif
 
 typedef struct {
-	stress_args_t *args;	/* args info */
+	stress_args_t *args;		/* args info */
 	size_t instance;		/* per thread instance number */
+	int rc;				/* return status */
 } stress_malloc_args_t;
 
 static const stress_help_t help[] = {
@@ -98,66 +99,6 @@ static inline ALWAYS_INLINE void stress_alloc_action(const char *str, const size
 	alloc_size = size;
 }
 
-static int stress_set_malloc_mlock(const char *opt)
-{
-	return stress_set_setting_true("malloc-mlock", opt);
-}
-
-static int stress_set_malloc_bytes(const char *opt)
-{
-	size_t bytes;
-
-	bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("malloc-bytes", bytes,
-		MIN_MALLOC_BYTES, MAX_MALLOC_BYTES);
-	return stress_set_setting("malloc-bytes", TYPE_ID_SIZE_T, &bytes);
-}
-
-static int stress_set_malloc_max(const char *opt)
-{
-	size_t max;
-
-	max = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range("malloc-max", max,
-		MIN_MALLOC_MAX, MAX_MALLOC_MAX);
-	return stress_set_setting("malloc-max", TYPE_ID_SIZE_T, &max);
-}
-
-static int stress_set_malloc_threshold(const char *opt)
-{
-	size_t threshold;
-
-	threshold = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range("malloc-threshold", threshold,
-		MIN_MALLOC_THRESHOLD, MAX_MALLOC_THRESHOLD);
-	return stress_set_setting("malloc-threshold", TYPE_ID_SIZE_T, &threshold);
-}
-
-static int stress_set_malloc_pthreads(const char *opt)
-{
-	size_t npthreads;
-
-	npthreads = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range("malloc-pthreads", npthreads,
-		0, MAX_MALLOC_PTHREADS);
-	return stress_set_setting("malloc-pthreads", TYPE_ID_SIZE_T, &npthreads);
-}
-
-static int stress_set_malloc_touch(const char *opt)
-{
-	return stress_set_setting_true("malloc-touch", opt);
-}
-
-static int stress_set_malloc_trim(const char *opt)
-{
-	return stress_set_setting_true("malloc-trim", opt);
-}
-
-static int stress_set_malloc_zerofree(const char *opt)
-{
-	return stress_set_setting_true("malloc-zerofree", opt);
-}
-
 /*
  *  stress_malloc_free()
  *	standard free, ignore length
@@ -212,7 +153,7 @@ static void stress_malloc_page_touch(
 
 static void *stress_malloc_loop(void *ptr)
 {
-	const stress_malloc_args_t *malloc_args = (stress_malloc_args_t *)ptr;
+	stress_malloc_args_t *malloc_args = (stress_malloc_args_t *)ptr;
 	register stress_malloc_info_t *info;
 	stress_args_t *args = malloc_args->args;
 	const size_t page_size = args->page_size;
@@ -238,6 +179,7 @@ static void *stress_malloc_loop(void *ptr)
 	if (info == MAP_FAILED) {
 		pr_inf("%s: cannot mmap address buffer of size %zd bytes: %d (%s)\n",
 			args->name, info_size, errno, strerror(errno));
+		malloc_args->rc = EXIT_FAILURE;
 		return &nowt;
 	}
 	for (;;) {
@@ -269,6 +211,8 @@ static void *stress_malloc_loop(void *ptr)
 				if (UNLIKELY(verify && (uintptr_t)info[i].addr != *info[i].addr)) {
 					pr_fail("%s: allocation at %p does not contain correct value\n",
 						args->name, (void *)info[i].addr);
+					malloc_args->rc = EXIT_FAILURE;
+					break;
 				}
 				stress_alloc_action("free", info[i].len);
 				free_func(info[i].addr, info[i].len);
@@ -292,6 +236,8 @@ static void *stress_malloc_loop(void *ptr)
 					if (UNLIKELY(verify && (uintptr_t)info[i].addr != *info[i].addr)) {
 						pr_fail("%s: allocation at %p does not contain correct value\n",
 							args->name, (void *)info[i].addr);
+						malloc_args->rc = EXIT_FAILURE;
+						break;
 					}
 					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 						break;
@@ -359,6 +305,21 @@ static void *stress_malloc_loop(void *ptr)
 					info[i].len = len;
 					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, true)))
 						break;
+
+#if defined(HAVE_MALLOC_USABLE_SIZE)
+					/* add some sanity checking */
+					if (UNLIKELY(verify)) {
+						const size_t usable_size = malloc_usable_size(info[i].addr);
+
+						if (usable_size < len) {
+							pr_fail("%s: malloc_usable_size on %p returned a "
+								"value %zu, expected %zu or larger\n",
+								args->name, info[i].addr, usable_size, len);
+							malloc_args->rc = EXIT_FAILURE;
+							break;
+						}
+					}
+#endif
 				} else {
 					info[i].len = 0;
 				}
@@ -376,6 +337,7 @@ static void *stress_malloc_loop(void *ptr)
 		if (verify && info[j].addr && ((uintptr_t)info[j].addr != *info[j].addr)) {
 			pr_fail("%s: allocation at %p does not contain correct value\n",
 				args->name, (void *)info[j].addr);
+			malloc_args->rc = EXIT_FAILURE;
 		}
 		stress_alloc_action("free", info[j].len);
 		free_func(info[j].addr, info[j].len);
@@ -399,6 +361,7 @@ static void MLOCKED_TEXT stress_malloc_sigsegv_handler(int signum)
 static int stress_malloc_child(stress_args_t *args, void *context)
 {
 	int ret;
+	NOCLOBBER int rc = EXIT_SUCCESS;
 	/*
 	 *  pthread instance 0 is actually the main child process,
 	 *  insances 1..N are pthreads 0..N-1
@@ -433,9 +396,12 @@ static int stress_malloc_child(stress_args_t *args, void *context)
 
 	malloc_args[0].args = args;
 	malloc_args[0].instance = 0;
+	malloc_args[0].rc = EXIT_SUCCESS;
 
 	(void)context;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 #if defined(HAVE_LIB_PTHREAD)
@@ -444,19 +410,24 @@ static int stress_malloc_child(stress_args_t *args, void *context)
 	for (j = 0; j < malloc_pthreads; j++) {
 		malloc_args[j + 1].args = args;
 		malloc_args[j + 1].instance = j + 1;
+		malloc_args[j + 1].rc = EXIT_SUCCESS;
 		pthreads[j].ret = pthread_create(&pthreads[j].pthread, NULL,
-			stress_malloc_loop, (void *)&malloc_args);
+			stress_malloc_loop, (void *)&malloc_args[j + 1]);
 	}
 #else
 	if ((args->instance == 0) && (malloc_pthreads > 0))
 		pr_inf("%s: pthreads not supported, ignoring the "
 			"--malloc-pthreads option\n", args->name);
 #endif
-	stress_malloc_loop(&malloc_args);
+	stress_malloc_loop(&malloc_args[0]);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if defined(HAVE_LIB_PTHREAD)
 	keep_thread_running_flag = false;
+
+	if (malloc_args[0].rc == EXIT_FAILURE)
+		rc = EXIT_FAILURE;
+
 	for (j = 0; j < malloc_pthreads; j++) {
 		if (pthreads[j].ret)
 			continue;
@@ -466,9 +437,11 @@ static int stress_malloc_child(stress_args_t *args, void *context)
 			pr_fail("%s: pthread_join failed (parent), errno=%d (%s)\n",
 				args->name, ret, strerror(ret));
 		}
+		if (malloc_args[j].rc == EXIT_FAILURE)
+			rc = EXIT_FAILURE;
 	}
 #endif
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
@@ -483,7 +456,7 @@ static int stress_malloc(stress_args_t *args)
 
 	stress_alloc_action("<unknown>", 0);
 
-	counter_lock = stress_lock_create();
+	counter_lock = stress_lock_create("counter");
 	if (!counter_lock) {
 		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
@@ -535,22 +508,22 @@ static int stress_malloc(stress_args_t *args)
 	return ret;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_malloc_bytes,	stress_set_malloc_bytes },
-	{ OPT_malloc_max,	stress_set_malloc_max },
-	{ OPT_malloc_mlock,	stress_set_malloc_mlock },
-	{ OPT_malloc_pthreads,	stress_set_malloc_pthreads },
-	{ OPT_malloc_threshold,	stress_set_malloc_threshold },
-	{ OPT_malloc_touch,	stress_set_malloc_touch },
-	{ OPT_malloc_trim,	stress_set_malloc_trim },
-	{ OPT_malloc_zerofree,	stress_set_malloc_zerofree },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_malloc_bytes,	"malloc-bytes",     TYPE_ID_SIZE_T_BYTES, MIN_MALLOC_BYTES, MAX_MALLOC_BYTES, NULL },
+	{ OPT_malloc_max,	"malloc-max",       TYPE_ID_SIZE_T_BYTES, MIN_MALLOC_MAX, MAX_MALLOC_MAX, NULL },
+	{ OPT_malloc_mlock,	"malloc-mlock",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_malloc_pthreads,	"malloc-pthreads",  TYPE_ID_SIZE_T, 0, MAX_MALLOC_PTHREADS, NULL },
+	{ OPT_malloc_threshold,	"malloc-thresh",    TYPE_ID_SIZE_T_BYTES, MIN_MALLOC_THRESHOLD, MAX_MALLOC_THRESHOLD, NULL },
+	{ OPT_malloc_touch,	"malloc-touch",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_malloc_trim,	"malloc-trim",      TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_malloc_zerofree,	"malloc-zerofree",  TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 stressor_info_t stress_malloc_info = {
 	.stressor = stress_malloc,
 	.class = CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
