@@ -34,23 +34,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		   	NULL }
 };
 
-/*
- *  stress_set_mlockmany_procs()
- *      set number of processes to spawn to mlock pages
- */
-static int stress_set_mlockmany_procs(const char *opt)
-{
-	size_t mlockmany_procs;
-
-	mlockmany_procs = (size_t)stress_get_uint64(opt);
-	stress_check_range("mlockmany-procs", mlockmany_procs,
-		1, 1000000);
-	return stress_set_setting("mlockmany-procs", TYPE_ID_SIZE_T, &mlockmany_procs);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_mlockmany_procs,	stress_set_mlockmany_procs },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_mlockmany_procs, "mlockmany-procs", TYPE_ID_SIZE_T, 1, 1000000, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_MLOCK)
@@ -139,8 +125,10 @@ static int stress_mlockmany(stress_args_t *args)
 #else
 	mlock_size = args->page_size * 1024;
 #endif
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		unsigned int n;
@@ -264,7 +252,7 @@ unmap:
 stressor_info_t stress_mlockmany_info = {
 	.stressor = stress_mlockmany,
 	.class = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
 
@@ -273,7 +261,7 @@ stressor_info_t stress_mlockmany_info = {
 stressor_info_t stress_mlockmany_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without mlock() support"
 };
