@@ -21,10 +21,19 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 
+#define MIN_MMAPFORK_BYTES     (4 * KB)
+#define MAX_MMAPFORK_BYTES     (MAX_MEM_LIMIT)
+
 static const stress_help_t help[] = {
-	{ NULL,	"mmapfork N",	  "start N workers stressing many forked mmaps/munmaps" },
-	{ NULL,	"mmapfork-ops N", "stop after N mmapfork bogo operations" },
-	{ NULL,	NULL,		  NULL }
+	{ NULL,	"mmapfork N",	    "start N workers stressing many forked mmaps/munmaps" },
+	{ NULL,	"mmapfork-ops N",   "stop after N mmapfork bogo operations" },
+	{ NULL,	"mmapfork-bytes N", "mmap and munmap N bytes by workers for each stress iteration" },
+	{ NULL,	NULL,		    NULL }
+};
+
+static const stress_opt_t opts[] = {
+	{ OPT_mmapfork_bytes, "mmapfork-bytes", TYPE_ID_SIZE_T_BYTES, MIN_MMAPFORK_BYTES, MAX_MMAPFORK_BYTES, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SYS_SYSINFO_H) &&	\
@@ -101,7 +110,7 @@ static bool stress_memory_is_not_zero(const uint8_t *ptr, const size_t size)
  */
 static int stress_mmapfork(stress_args_t *args)
 {
-	pid_t pids[MAX_PIDS];
+	stress_pid_t s_pids[MAX_PIDS];
 	struct sysinfo info;
 	void *ptr;
 	uint64_t segv_count = 0;
@@ -119,28 +128,32 @@ static int stress_mmapfork(stress_args_t *args)
 	wipe_ptr = mmap(NULL, wipe_size, PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (wipe_ptr != MAP_FAILED) {
+		stress_set_vma_anon_name(wipe_ptr, wipe_size, "wipe-on-fork-data");
 		(void)shim_memset(wipe_ptr, 0xff, wipe_size);
 		if (shim_madvise(wipe_ptr, wipe_size, MADV_WIPEONFORK) == 0)
 			wipe_ok = true;
 	}
 #endif
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		size_t i, len;
 
 		for (i = 0; i < MAX_PIDS; i++)
-			pids[i] = -1;
+			s_pids[i].pid = -1;
 
 		for (i = 0; i < MAX_PIDS; i++) {
 			if (!stress_continue(args))
 				goto reap;
 
-			pids[i] = fork();
+			s_pids[i].pid = fork();
 			/* Out of resources for fork?, do a reap */
-			if (pids[i] < 0)
+			if (s_pids[i].pid < 0)
 				break;
-			if (pids[i] == 0) {
+			if (s_pids[i].pid == 0) {
 				/* Child */
 				const pid_t ppid = getppid();
 
@@ -156,6 +169,9 @@ static int stress_mmapfork(stress_args_t *args)
 						args->name, errno, strerror(errno));
 					_exit(MMAPFORK_FAILURE);
 				}
+
+				len = ((size_t)info.freeram / (args->num_instances * MAX_PIDS)) / 2;
+
 #if defined(MADV_WIPEONFORK)
 				if (wipe_ok && (wipe_ptr != MAP_FAILED) &&
 				    stress_memory_is_not_zero(wipe_ptr, wipe_size)) {
@@ -165,7 +181,14 @@ static int stress_mmapfork(stress_args_t *args)
 				}
 #endif
 
-				len = ((size_t)info.freeram / (args->num_instances * MAX_PIDS)) / 2;
+				if (!stress_get_setting("mmapfork-bytes", &len)) {
+					if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+						len = MIN_MMAPFORK_BYTES;
+				}
+
+				if (len < MIN_MMAPFORK_BYTES)
+					len = MIN_MMAPFORK_BYTES;
+
 				segv_ret = MMAPFORK_SEGV_MMAP;
 				ptr = stress_mmap_populate(NULL, len, PROT_READ | PROT_WRITE,
 					MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -203,10 +226,10 @@ static int stress_mmapfork(stress_args_t *args)
 		for (i = 0; i < MAX_PIDS; i++) {
 			int status;
 
-			if (UNLIKELY(pids[i] < 0))
+			if (UNLIKELY(s_pids[i].pid < 0))
 				continue;
 
-			if (shim_waitpid(pids[i], &status, 0) < 0) {
+			if (shim_waitpid(s_pids[i].pid, &status, 0) < 0) {
 				if (errno != EINTR) {
 					pr_err("%s: waitpid errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -215,7 +238,7 @@ static int stress_mmapfork(stress_args_t *args)
 					goto reap;
 				}
 			} else {
-				pids[i] = -1;
+				s_pids[i].pid = -1;
 				if (WIFEXITED(status)) {
 					int masked = WEXITSTATUS(status) & MMAPFORK_MASK;
 
@@ -227,7 +250,7 @@ static int stress_mmapfork(stress_args_t *args)
 			}
 		}
 reap:
-		stress_kill_and_wait_many(args, pids, MAX_PIDS, SIGALRM, false);
+		stress_kill_and_wait_many(args, s_pids, MAX_PIDS, SIGALRM, false);
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
@@ -262,9 +285,11 @@ reap:
 	return EXIT_SUCCESS;
 }
 
+
 stressor_info_t stress_mmapfork_info = {
 	.stressor = stress_mmapfork,
 	.class = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -273,6 +298,7 @@ stressor_info_t stress_mmapfork_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_VM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without sys/sysinfo_h or sysinfo() system call"
 };
