@@ -41,20 +41,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-static int stress_set_mq_size(const char *opt)
-{
-	uint64_t sz;
-	int mq_size;
-
-	sz = stress_get_uint64(opt);
-	stress_check_range("mq-size", sz, MIN_MQ_SIZE, MAX_MQ_SIZE);
-	mq_size = (int)sz;
-	return stress_set_setting("mq-size", TYPE_ID_INT, &mq_size);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_mq_size,	stress_set_mq_size },
-	{ 0,		NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_mq_size, "mq-size", TYPE_ID_INT, MIN_MQ_SIZE, MAX_MQ_SIZE, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_MQUEUE_H) &&	\
@@ -218,9 +207,9 @@ static int stress_mq(stress_args_t *args)
 		abs_timeout.tv_nsec = 0;
 	}
 
-
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
@@ -258,7 +247,8 @@ again:
 					char buffer[1024];
 					struct stat statbuf;
 					void *ptr;
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 					struct pollfd fds[1];
 #endif
 					/* On Linux, one can seek on a mq descriptor */
@@ -271,7 +261,8 @@ again:
 					ptr = mmap(NULL, 16, PROT_READ, MAP_SHARED, mq, 0);
 					if (ptr != MAP_FAILED)
 						(void)munmap(ptr, 16);
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 					/* ..and poll too */
 					fds[0].fd = mq;
 					fds[0].events = POLLIN;
@@ -535,16 +526,16 @@ finish:
 
 stressor_info_t stress_mq_info = {
 	.stressor = stress_mq,
-	.class = CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
 stressor_info_t stress_mq_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_SCHEDULER | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without mqueue.h, POSIX message queues or librt support"
