@@ -73,16 +73,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_nanosleep_threads(const char *opt)
-{
-	uint32_t nanosleep_threads;
-
-	nanosleep_threads = stress_get_uint32(opt);
-	stress_check_range("nanosleep_threads", nanosleep_threads,
-		MIN_NANOSLEEP_THREADS, MAX_NANOSLEEP_THREADS);
-	return stress_set_setting("nanosleep-threads", TYPE_ID_UINT32, &nanosleep_threads);
-}
-
 static const stress_nanosleep_method_t stress_nanosleep_methods[] = {
 	{ "all",	STRESS_NANOSLEEP_ALL },
 	{ "cstate",	STRESS_NANOSLEEP_CSTATE },
@@ -92,34 +82,15 @@ static const stress_nanosleep_method_t stress_nanosleep_methods[] = {
 	{ "ms",		STRESS_NANOSLEEP_MS },
 };
 
-/*
- *  stress_set_nanosleep_method()
- *	set the default nanosleep time method
- */
-static int stress_set_nanosleep_method(const char *name)
+static const char *stress_nanosleep_method(const size_t i)
 {
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_nanosleep_methods); i++) {
-		if (!strcmp(stress_nanosleep_methods[i].name, name)) {
-			stress_set_setting("stress-nanosleep-method", TYPE_ID_INT, &stress_nanosleep_methods[i].mask);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "nanosleep-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_nanosleep_methods); i++) {
-		(void)fprintf(stderr, " %s", stress_nanosleep_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
+	return (i < SIZEOF_ARRAY(stress_nanosleep_methods)) ? stress_nanosleep_methods[i].name : NULL;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_nanosleep_threads,	stress_set_nanosleep_threads },
-	{ OPT_nanosleep_method,		stress_set_nanosleep_method },
-	{ 0,				NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_nanosleep_threads, "nanosleep-threads", TYPE_ID_UINT32, MIN_NANOSLEEP_THREADS, MAX_NANOSLEEP_THREADS, NULL },
+	{ OPT_nanosleep_method,  "nanosleep-method",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_nanosleep_method },
+	END_OPT,
 };
 
 #if defined(HAVE_LIB_PTHREAD)
@@ -191,7 +162,6 @@ static void *stress_nanosleep_pthread(void *c)
 	while (stress_continue(args) &&
 	       !thread_terminate &&
 	       (!ctxt->max_ops || (ctxt->counter < ctxt->max_ops))) {
-		register unsigned long i;
 		cpu_cstate_t *cc;
 
 		if (ctxt->mask & STRESS_NANOSLEEP_CSTATE) {
@@ -201,9 +171,11 @@ static void *stress_nanosleep_pthread(void *c)
 			}
 		}
 		if (ctxt->mask & STRESS_NANOSLEEP_RANDOM) {
+			register unsigned long i;
+
 			for (i = 1 << 18; i > 0; i >>=1) {
 				register const unsigned long mask = (i - 1);
-				long nsec = (long)(stress_mwc32() & mask) + 8;
+				register const long nsec = (long)(stress_mwc32() & mask) + 8;
 
 				if (stress_nanosleep_ns(ctxt, nsec) < 0)
 					break;
@@ -249,7 +221,7 @@ static int stress_nanosleep(stress_args_t *args)
 	if (mask & STRESS_NANOSLEEP_CSTATE) {
 		if (cstate_list == NULL) {
 			if (args->instance == 0)
-				pr_inf("%s: no C states found, using just random nanosleeps instead\n", args->name);
+				pr_inf("%s: no C states found, using random nanosleeps instead\n", args->name);
 			mask = STRESS_NANOSLEEP_RANDOM;
 		} else {
 			if ((args->instance == 0) &&
@@ -308,6 +280,8 @@ static int stress_nanosleep(stress_args_t *args)
 			goto tidy;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -364,11 +338,11 @@ tidy:
 	overrun_nsec -= overhead_nsec;
 	overrun_nsec = (overrun_count > 0.0) ? overrun_nsec / overrun_count : 0.0;
 	stress_metrics_set(args, 0, "nanosec sleep overrun",
-		overrun_nsec, STRESS_GEOMETRIC_MEAN);
+		overrun_nsec, STRESS_METRIC_GEOMETRIC_MEAN);
 	underrun_nsec -= overhead_nsec;
 	underrun_nsec = (underrun_count > 0.0) ? underrun_nsec / underrun_count : 0.0;
 	stress_metrics_set(args, 1, "nanosec sleep underrun",
-		underrun_nsec, STRESS_GEOMETRIC_MEAN);
+		underrun_nsec, STRESS_METRIC_GEOMETRIC_MEAN);
 #endif
 
 	if (limited) {
@@ -388,7 +362,7 @@ tidy:
 stressor_info_t stress_nanosleep_info = {
 	.stressor = stress_nanosleep,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -396,7 +370,7 @@ stressor_info_t stress_nanosleep_info = {
 stressor_info_t stress_nanosleep_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread, librt or nanosleep() system call support"
