@@ -213,6 +213,12 @@ static int stress_netlink_proc(stress_args_t *args)
 	addr.nl_groups = CN_IDX_PROC;
 
 	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+		if (errno == EPERM) {
+			pr_inf_skip("%s: bind failed, no permission, "
+				"skipping stressor\n", args->name);
+			(void)close(sock);
+			return EXIT_NO_RESOURCE;
+		}
 		pr_err("%s: bind failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(sock);
@@ -250,8 +256,9 @@ static int stress_netlink_proc(stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		spawn_several(args->name, 0, 5);
