@@ -174,7 +174,7 @@ static int stress_oom_pipe_child(stress_args_t *args, void *ctxt)
 	/* And close the pipes */
 clean:
 	for (i = 0, fd = fds; i < max_pipes * 2; i++, fd++) {
-		if (*fd >= 0)
+		if (*fd != -1)
 			(void)close(*fd);
 	}
 	return EXIT_SUCCESS;
@@ -198,6 +198,7 @@ static int stress_oom_pipe(stress_args_t *args)
 		pr_inf_skip("%s: cannot allocate pipe write buffer, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buffer, buffer_size, "rw-pipe-buffer");
 	context.wr_buffer = (uint32_t *)buffer;
 	context.rd_buffer = (uint32_t *)((uintptr_t)buffer + page_size);
 
@@ -206,16 +207,23 @@ static int stress_oom_pipe(stress_args_t *args)
 
 	context.fds = calloc(context.max_fd, sizeof(*context.fds));
 	if (!context.fds) {
-		pr_inf_skip("%s: cannot allocate %zd file descriptors, skipping stressor\n",
-			args->name, context.max_fd);
-		(void)munmap(buffer, buffer_size);
-		return EXIT_NO_RESOURCE;
+		/* Shrink down */
+		context.max_fd = 1024 * 1024;
+		context.fds = calloc(context.max_fd, sizeof(*context.fds));
+		if (!context.fds) {
+			pr_inf_skip("%s: cannot allocate %zd file descriptors, skipping stressor\n",
+				args->name, context.max_fd);
+			(void)munmap(buffer, buffer_size);
+			return EXIT_NO_RESOURCE;
+		}
 	}
 
 	if (context.max_pipe_size < page_size)
 		context.max_pipe_size = page_size;
 	context.max_pipe_size &= ~(page_size - 1);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, &context, stress_oom_pipe_child, STRESS_OOMABLE_DROP_CAP);
