@@ -24,6 +24,8 @@
 #include "core-killpid.h"
 #include "core-pragma.h"
 
+#include <math.h>
+
 #if defined(HAVE_LINUX_AUDIT_H)
 #include <linux/audit.h>
 #endif
@@ -56,6 +58,7 @@ static const stress_help_t help[] = {
 #if defined(__NR_rt_sigreturn)	&&	\
     defined(__NR_rt_sigprocmask)
 #define STRESS_OPCODE_USE_SIGLONGJMP
+static bool jmp_env_set;
 static sigjmp_buf jmp_env;
 #endif
 
@@ -158,7 +161,7 @@ static struct sock_fprog prog = {
 };
 #endif
 
-static void MLOCKED_TEXT stress_badhandler(int signum)
+static void MLOCKED_TEXT stress_opcode_child_sighandler(int signum)
 {
 	if ((signum >= 0) && (signum < MAX_SIGS)) {
 		volatile stress_opcode_state_t *vstate = (volatile stress_opcode_state_t *)state;
@@ -185,7 +188,8 @@ static void MLOCKED_TEXT stress_badhandler(int signum)
 	 *  try to continue for less significant signals
 	 *  such as SIGFPE and SIGTRAP
 	 */
-	siglongjmp(jmp_env, 1);
+	if (jmp_env_set)
+		siglongjmp(jmp_env, 1);
 #else
 	_exit(1);
 #endif
@@ -222,7 +226,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 {
 	switch (STRESS_OPCODE_SIZE) {
 	case 8:	{
-			register uint8_t tmp8 = *op & 0xff;
+			register const uint8_t tmp8 = *op & 0xff;
 			register uint8_t *ops = (uint8_t *)ops_begin;
 			register ssize_t i = (ssize_t)page_size;
 
@@ -232,7 +236,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 		}
 		break;
 	case 16: {
-			register uint16_t tmp16 = *op & 0xffff;
+			register const uint16_t tmp16 = *op & 0xffff;
 			register uint16_t *ops = (uint16_t *)ops_begin;
 			register ssize_t i = (ssize_t)(page_size >> 1);
 
@@ -243,7 +247,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 		break;
 	default:
 	case 32: {
-			register uint32_t tmp32 = *op & 0xffffffffL;
+			register const uint32_t tmp32 = *op & 0xffffffffL;
 			register uint32_t *ops = (uint32_t *)ops_begin;
 			register size_t i = (ssize_t)(page_size >> 2);
 
@@ -254,7 +258,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 		break;
 	case 48:
 		{
-			register uint64_t tmp64 = *op;
+			register const uint64_t tmp64 = *op;
 			register uint8_t *ops = (uint8_t *)ops_begin;
 			register size_t i = (ssize_t)(page_size / 6);
 
@@ -272,7 +276,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 		}
 		break;
 	case 64: {
-			register uint64_t tmp64 = *op;
+			register const uint64_t tmp64 = *op;
 			register uint64_t *ops = (uint64_t *)ops_begin;
 			register size_t i = (ssize_t)(page_size >> 3);
 
@@ -316,10 +320,18 @@ static void stress_opcode_text(
 {
 	char *text_start, *text_end;
 	const size_t ops_len = (uintptr_t)ops_end - (uintptr_t)ops_begin;
-	const size_t text_len = stress_exec_text_addr(&text_start, &text_end) - 8;
+	size_t text_len = stress_exec_text_addr(&text_start, &text_end);
 	uint8_t *ops;
 	size_t offset;
 
+	/*
+	 *  Don't access last 8 bytes to avoid 64 bit accesses off
+	 *  the end.
+	 */
+	if (text_len <= 8)
+		return;
+	text_len -= 8;
+
 	if (text_len < ops_len) {
 		stress_opcode_random(page_size, ops_begin, ops_end, op);
 		return;
@@ -328,11 +340,12 @@ static void stress_opcode_text(
 	offset = stress_mwc64modn(text_len - ops_len) & ~(0x7ULL);
 	(void)shim_memcpy(ops_begin, text_start + offset, ops_len);
 	for (ops = (uint8_t *)ops_begin; ops < (const uint8_t *)ops_end; ops++) {
-		const uint8_t rnd = stress_mwc8();
+		register const uint8_t rnd = stress_mwc8();
 
 		/* 1 in 8 chance of random bit corruption */
 		if (rnd < 32) {
-			uint8_t bit = (uint8_t)(1 << (rnd & 7));
+			register const uint8_t bit = (uint8_t)(1 << (rnd & 7));
+
 			*ops ^= bit;
 		}
 	}
@@ -345,30 +358,6 @@ static const stress_opcode_method_info_t stress_opcode_methods[] = {
 	{ "mixed",	stress_opcode_mixed },
 };
 
-/*
- *  stress_set_opcode_method()
- *      set default opcode stress method
- */
-static int stress_set_opcode_method(const char *name)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_opcode_methods); i++) {
-		if (!strcmp(stress_opcode_methods[i].name, name)) {
-			stress_set_setting("opcode-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "opcode-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_opcode_methods); i++) {
-		(void)fprintf(stderr, " %s", stress_opcode_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
-
 /*
  *  stress_opcode
  *	stress with random opcodes
@@ -407,6 +396,7 @@ static int stress_opcode(stress_args_t *args)
 			args->name, args->page_size, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(state, sizeof(*state), "state");
 	vstate = (volatile stress_opcode_state_t *)state;
 
 	opcodes = (void *)stress_mmap_populate(NULL, page_size * PAGES,
@@ -418,12 +408,15 @@ static int stress_opcode(stress_args_t *args)
 		(void)munmap((void *)state, sizeof(*state));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(opcodes, page_size * PAGES, "opcodes");
 	/* Force pages resident */
 	(void)shim_memset(opcodes, 0x00, page_size * PAGES);
 
 	(void)stress_get_setting("opcode-method", &opcode_method);
 	method = &stress_opcode_methods[opcode_method];
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	op_start = (num_opcodes * (double)args->instance) / args->num_instances;
@@ -446,6 +439,7 @@ static int stress_opcode(stress_args_t *args)
 			stress_set_proc_name(buf);
 		}
 again:
+		jmp_env_set = false;
 		pid = fork();
 		if (pid < 0) {
 			if (stress_redo_fork(args, errno))
@@ -465,6 +459,11 @@ again:
 
 			(void)sched_settings_apply(true);
 
+			for (i = 0; i < SIZEOF_ARRAY(sigs); i++) {
+				if (stress_sighandler(args->name, sigs[i], stress_opcode_child_sighandler, NULL) < 0)
+					_exit(EXIT_FAILURE);
+			}
+
 			/* We don't want bad ops clobbering this region */
 			stress_shared_readonly();
 
@@ -475,10 +474,6 @@ again:
 			if (stress_drop_capabilities(args->name) < 0) {
 				_exit(EXIT_NO_RESOURCE);
 			}
-			for (i = 0; i < SIZEOF_ARRAY(sigs); i++) {
-				if (stress_sighandler(args->name, sigs[i], stress_badhandler, NULL) < 0)
-					_exit(EXIT_FAILURE);
-			}
 
 			(void)mprotect((void *)opcodes, page_size, PROT_NONE);
 			(void)mprotect((void *)ops_end, page_size, PROT_NONE);
@@ -529,6 +524,7 @@ again:
 				ret = sigsetjmp(jmp_env, 1);
 				if (ret == 1)
 					goto exercise;
+				jmp_env_set = true;
 			}
 #endif
 #if defined(HAVE_LINUX_SECCOMP_H) &&	\
@@ -594,34 +590,34 @@ finish:
 				args->name, estimated_duration);
 		} else {
 			pr_dbg("%s: estimated time to cover all op-codes: %s\n",
-				args->name, stress_duration_to_str(estimated_duration, false));
+				args->name, stress_duration_to_str(estimated_duration, false, false));
 		}
 	}
 
 	rate = (duration > 0.0) ? (double)vstate->ops_attempted / duration : 0.0;
 	stress_metrics_set(args, 0, "opcodes exercised per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->ops_ok / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 1, "% opcodes successfully executed",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGILL] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 2, "% illegal opcodes executed",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGBUS] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 3, "% opcodes generated SIGBUS",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGSEGV] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 4, "% opcodes generated SIGSEGV",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGFPE] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 5, "% opcodes generated SIGFPE",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGTRAP] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 6, "% opcodes generated SIGTRAP",
-		percent, STRESS_GEOMETRIC_MEAN);
+		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	rate = (duration > 0.0) ? (double)forks / duration : 0.0;
 	stress_metrics_set(args, 7, "forks per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -630,43 +626,33 @@ err:
 	return rc;
 }
 
-static void stress_opcode_set_default(void)
+static const char *stress_opcode_method(const size_t i)
 {
-	stress_set_opcode_method("random");
+	return (i < SIZEOF_ARRAY(stress_opcode_methods)) ? stress_opcode_methods[i].name : NULL;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_opcode_method,	stress_set_opcode_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_opcode_method, "opcode-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_opcode_method },
+	END_OPT,
 };
 
 stressor_info_t stress_opcode_info = {
 	.stressor = stress_opcode,
-	.set_default = stress_opcode_set_default,
 	.class = CLASS_CPU | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
 #else
 
-static int stress_set_opcode_method(const char *name)
-{
-	(void)name;
-
-	(void)fprintf(stderr, "opcode-method not implemented");
-
-	return -1;
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_opcode_method,	stress_set_opcode_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_opcode_method, "opcode-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_unimplemented_method },
+	END_OPT,
 };
 
 stressor_info_t stress_opcode_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without linux/seccomp.h, linux/audit.h, linux/filter.h, sys/prctl.h or mprotect()"
 };
