@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 #define DEFAULT_PAGE_MOVE_BYTES		(4 * MB)
 #define MIN_PAGE_MOVE_BYTES		(64 * KB)
@@ -35,25 +36,10 @@ typedef struct {
 	size_t	page_num;		/* original page number relative to start of entire mapping */
 } page_info_t;
 
-static int stress_set_pagemove_mlock(const char *opt)
-{
-	return stress_set_setting_true("pagemove-mlock", opt);
-}
-
-static int stress_set_pagemove_bytes(const char *opt)
-{
-	size_t pagemove_bytes;
-
-	pagemove_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("pagemove-bytes", pagemove_bytes,
-		MIN_PAGE_MOVE_BYTES, MAX_PAGE_MOVE_BYTES);
-	return stress_set_setting("pagemove-bytes", TYPE_ID_SIZE_T, &pagemove_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_pagemove_bytes,	stress_set_pagemove_bytes },
-	{ OPT_pagemove_mlock,	stress_set_pagemove_mlock },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_pagemove_bytes, "pagemove-bytes", TYPE_ID_SIZE_T_BYTES, MIN_PAGE_MOVE_BYTES, MAX_PAGE_MOVE_BYTES, NULL },
+	{ OPT_pagemove_mlock, "pagemove-mlock", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_MREMAP) &&	\
@@ -65,7 +51,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *	report remap failure message
  */
 static void stress_pagemove_remap_fail(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const uint8_t *from,
 	const uint8_t *to)
 {
@@ -73,10 +59,9 @@ static void stress_pagemove_remap_fail(
 		args->name, from, to, errno, strerror(errno));
 }
 
-static int stress_pagemove_child(const stress_args_t *args, void *context)
+static int stress_pagemove_child(stress_args_t *args, void *context)
 {
 	size_t sz, pages, pagemove_bytes = DEFAULT_PAGE_MOVE_BYTES;
-	int flags = MAP_PRIVATE | MAP_ANONYMOUS;
 	const size_t page_size = args->page_size;
 	size_t page_num;
 	uint8_t *buf, *buf_end, *unmapped_page = NULL, *ptr;
@@ -104,7 +89,10 @@ static int stress_pagemove_child(const stress_args_t *args, void *context)
 	sz = pagemove_bytes & ~(page_size - 1);
 	pages = sz / page_size;
 
-	buf = (uint8_t *)mmap(NULL, sz + page_size, PROT_READ | PROT_WRITE, flags, -1, 0);
+	buf = (uint8_t *)stress_mmap_populate(NULL, sz + page_size,
+		PROT_READ | PROT_WRITE,
+		MAP_PRIVATE | MAP_ANONYMOUS,
+		-1, 0);
 	if (buf == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %zu bytes (errno=%d) %s, skipping stressor\n",
 			args->name, sz + page_size, errno, strerror(errno));
@@ -116,6 +104,8 @@ static int stress_pagemove_child(const stress_args_t *args, void *context)
 	unmapped_page = buf_end;
 	(void)munmap((void *)unmapped_page, page_size);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -245,7 +235,8 @@ fail:
 	(void)munmap(buf, sz);
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
-	stress_metrics_set(args, 0, "page remaps per sec", rate);
+	stress_metrics_set(args, 0, "page remaps per sec",
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	return rc;
 }
@@ -254,7 +245,7 @@ fail:
  *  stress_pagemove()
  *	stress mmap
  */
-static int stress_pagemove(const stress_args_t *args)
+static int stress_pagemove(stress_args_t *args)
 {
 	return stress_oomable_child(args, NULL, stress_pagemove_child, STRESS_OOMABLE_NORMAL);
 }
@@ -262,7 +253,7 @@ static int stress_pagemove(const stress_args_t *args)
 stressor_info_t stress_pagemove_info = {
 	.stressor = stress_pagemove,
 	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -270,7 +261,7 @@ stressor_info_t stress_pagemove_info = {
 stressor_info_t stress_pagemove_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without mremap() or MREMAP_FIXED/MREMAP_MAYMOVE defined"
