@@ -21,6 +21,8 @@
 #include "core-arch.h"
 #include "core-capabilities.h"
 
+#include <sys/ioctl.h>
+
 #if defined(HAVE_ASM_MTRR_H)
 #include <asm/mtrr.h>
 #endif
@@ -32,14 +34,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		  NULL }
 };
 
-static int stress_set_physpage_mtrr(const char *opt)
-{
-	return stress_set_setting_true("physpage-mtrr", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_physpage_mtrr,	stress_set_physpage_mtrr },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_physpage_mtrr, "physpage-mtrr", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(__linux__)
@@ -207,16 +204,19 @@ static int stress_virt_to_phys(
 				VOID_RET(ssize_t, read(fd_mem, data, sizeof(data)));
 			}
 
-			ptr = mmap(NULL, page_size, PROT_READ,
+			ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ,
 				MAP_SHARED, fd_mem, (off_t)phys_addr);
-			if (ptr != MAP_FAILED)
+			if (ptr != MAP_FAILED) {
+				stress_set_vma_anon_name(ptr, page_size, "ro-dev-mem");
 				(void)stress_munmap_retry_enomem((void *)ptr, page_size);
+			}
 			if (writable) {
-				ptr = mmap(NULL, page_size, PROT_READ | PROT_WRITE,
+				ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ | PROT_WRITE,
 					MAP_SHARED, fd_mem, (off_t)phys_addr);
 				if (ptr != MAP_FAILED) {
 					uint8_t val = *ptr;
 
+					stress_set_vma_anon_name(ptr, page_size, "rw-dev-mem");
 					*(volatile uint8_t *)ptr = val;
 					(void)stress_munmap_retry_enomem((void *)ptr, page_size);
 				}
@@ -285,14 +285,17 @@ static int stress_physpage(stress_args_t *args)
 	if (fd_mem < 0)
 		fd_mem = open("/dev/mem", O_RDONLY);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		void *nptr;
 
-		nptr = mmap(ptr, page_size, PROT_READ | PROT_WRITE,
-			MAP_POPULATE | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+		nptr = stress_mmap_populate((void *)ptr, page_size, PROT_READ | PROT_WRITE,
+					MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 		if (nptr != MAP_FAILED) {
+			stress_set_vma_anon_name(nptr, page_size, "rw-page");
 			(void)stress_virt_to_phys(args, page_size, fd_pm, fd_pc, fd_mem,
 				(uintptr_t)nptr, physpage_mtrr, true, &success);
 			(void)stress_munmap_retry_enomem(nptr, page_size);
@@ -306,9 +309,9 @@ static int stress_physpage(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (fd_mem > 0)
+	if (fd_mem >= 0)
 		(void)close(fd_mem);
-	if (fd_pc > 0)
+	if (fd_pc >= 0)
 		(void)close(fd_pc);
 	(void)close(fd_pm);
 
@@ -318,7 +321,7 @@ static int stress_physpage(stress_args_t *args)
 stressor_info_t stress_physpage_info = {
 	.stressor = stress_physpage,
 	.supported = stress_physpage_supported,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.class = CLASS_VM,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -326,7 +329,7 @@ stressor_info_t stress_physpage_info = {
 #else
 stressor_info_t stress_physpage_info = {
 	.stressor = stress_unimplemented,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.class = CLASS_VM,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
