@@ -38,8 +38,8 @@
 #endif
 
 static const stress_help_t help[] = {
-	{ NULL,	"procfs N",	"start N workers reading portions of /proc" },
-	{ NULL,	"procfs-ops N",	"stop procfs workers after N bogo read operations" },
+	{ NULL,	"prctl N",	"start N workers exercising prctl system call" },
+	{ NULL,	"prctls-ops N",	"stop prctl workers after N bogo prctl operations" },
 	{ NULL,	NULL,		NULL }
 };
 
@@ -1082,11 +1082,14 @@ static int stress_prctl(stress_args_t *args)
 {
 	void *page_anon;
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-	page_anon = stress_mmap_populate(NULL, args->page_size,	
+	page_anon = stress_mmap_populate(NULL, args->page_size,
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	do {
 		pid_t pid;
 again:
