@@ -80,6 +80,8 @@ static int OPTIMIZE3 stress_ptrace(stress_args_t *args)
 {
 	pid_t pid;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 again:
