@@ -61,20 +61,6 @@ static void MLOCKED_TEXT stress_qsort_handler(int signum)
 	}
 }
 
-/*
- *  stress_set_qsort_size()
- *	set qsort size
- */
-static int stress_set_qsort_size(const char *opt)
-{
-	uint64_t qsort_size;
-
-	qsort_size = stress_get_uint64(opt);
-	stress_check_range("qsort-size", qsort_size,
-		MIN_QSORT_SIZE, MAX_QSORT_SIZE);
-	return stress_set_setting("qsort-size", TYPE_ID_UINT64, &qsort_size);
-}
-
 typedef uint32_t qsort_swap_type_t;
 
 static inline size_t qsort_bm_minimum(const size_t x, const size_t y)
@@ -222,29 +208,11 @@ static const stress_qsort_method_t stress_qsort_methods[] = {
 	{ "qsort-bm",		qsort_bm },
 };
 
-static int stress_set_qsort_method(const char *opt)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_qsort_methods); i++) {
-		if (strcmp(opt, stress_qsort_methods[i].name) == 0) {
-			stress_set_setting("qsort-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "qsort-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_qsort_methods); i++) {
-		(void)fprintf(stderr, " %s", stress_qsort_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-	return -1;
-}
-
 static inline bool OPTIMIZE3 stress_qsort_verify_forward(
 	stress_args_t *args,
 	const int32_t *data,
-	const size_t n)
+	const size_t n,
+	int *rc)
 {
 	if (g_opt_flags & OPT_FLAGS_VERIFY) {
 		register const int32_t *ptr = data;
@@ -253,7 +221,7 @@ static inline bool OPTIMIZE3 stress_qsort_verify_forward(
 
 PRAGMA_UNROLL_N(8)
 		while (ptr < end) {
-			register int32_t next_val = *(ptr + 1);
+			register const int32_t next_val = *(ptr + 1);
 
 			if (UNLIKELY(val > next_val))
 				goto fail;
@@ -267,13 +235,15 @@ PRAGMA_UNROLL_N(8)
 fail:
 	pr_fail("%s: forward sort error detected, incorrect ordering found\n",
 		args->name);
+	*rc = EXIT_FAILURE;
 	return false;
 }
 
 static inline bool OPTIMIZE3 stress_qsort_verify_reverse(
 	stress_args_t *args,
 	const int32_t *data,
-	const size_t n)
+	const size_t n,
+	int *rc)
 {
 	if (g_opt_flags & OPT_FLAGS_VERIFY) {
 		register const int32_t *ptr = data;
@@ -296,6 +266,7 @@ PRAGMA_UNROLL_N(8)
 fail:
 	pr_fail("%s: reverse sort error detected, incorrect ordering found\n",
 		args->name);
+	*rc = EXIT_FAILURE;
 	return false;
 }
 
@@ -312,6 +283,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 	int ret;
 	double rate;
 	NOCLOBBER double duration = 0.0, count = 0.0, sorted = 0.0;
+	NOCLOBBER int rc = EXIT_SUCCESS;
 	qsort_func_t qsort_func;
 
 	stress_catch_sigill();
@@ -333,6 +305,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 			"skipping stressor\n", args->name, n, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(data, data_size, "qsort-data");
 
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
@@ -356,6 +329,8 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		pr_inf("%s: using method '%s'\n",
 			args->name, stress_qsort_methods[qsort_method].name);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -371,7 +346,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		count += (double)stress_sort_compare_get();
 		sorted += (double)n;
 
-		if (!stress_qsort_verify_forward(args, data, n))
+		if (!stress_qsort_verify_forward(args, data, n, &rc))
 			break;
 
 		if (!stress_continue_flag())
@@ -385,7 +360,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		count += (double)stress_sort_compare_get();
 		sorted += (double)n;
 
-		if (!stress_qsort_verify_reverse(args, data, n))
+		if (!stress_qsort_verify_reverse(args, data, n, &rc))
 			break;
 
 		if (!stress_continue_flag())
@@ -407,7 +382,7 @@ static int OPTIMIZE3 stress_qsort(stress_args_t *args)
 		count += (double)stress_sort_compare_get();
 		sorted += (double)n;
 
-		if (!stress_qsort_verify_reverse(args, data, n))
+		if (!stress_qsort_verify_reverse(args, data, n, &rc))
 			break;
 
 		stress_bogo_inc(args);
@@ -419,25 +394,30 @@ tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "qsort comparisons per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 	stress_metrics_set(args, 1, "qsort comparisons per item",
-		count / sorted, STRESS_HARMONIC_MEAN);
+		count / sorted, STRESS_METRIC_HARMONIC_MEAN);
 
 	(void)munmap((void *)data, data_size);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_qsort_integers,	stress_set_qsort_size },
-	{ OPT_qsort_method,	stress_set_qsort_method },
-	{ 0,			NULL }
+static const char *stress_qsort_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(stress_qsort_methods)) ? stress_qsort_methods[i].name : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_qsort_size,   "qsort-size",   TYPE_ID_UINT64, MIN_QSORT_SIZE, MAX_QSORT_SIZE, NULL },
+	{ OPT_qsort_method, "qsort-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_qsort_method },
+	END_OPT,
 };
 
 stressor_info_t stress_qsort_info = {
 	.stressor = stress_qsort,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SORT,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
