@@ -36,8 +36,6 @@
 
 #include <arpa/inet.h>
 
-#define MIN_RAWUDP_PORT		(1024)
-#define MAX_RAWUDP_PORT		(65535)
 #define DEFAULT_RAWUDP_PORT	(13000)
 
 #if !defined(SOL_UDP)
@@ -67,28 +65,10 @@ static int stress_rawudp_supported(const char *name)
 	return 0;
 }
 
-/*
- *  stress_set_rawudp_port()
- *	set port to use
- */
-static int stress_set_rawudp_port(const char *opt)
-{
-	int port;
-
-	stress_set_net_port("rawudp-port", opt,
-		MIN_RAWUDP_PORT, MAX_RAWUDP_PORT, &port);
-	return stress_set_setting("rawudp-port", TYPE_ID_INT, &port);
-}
-
-static int stress_set_rawudp_if(const char *name)
-{
-	return stress_set_setting("rawudp-if", TYPE_ID_STR, name);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_rawudp_port,	stress_set_rawudp_port },
-	{ OPT_rawudp_if,	stress_set_rawudp_if },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_rawudp_port, "rawudp-port", TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
+	{ OPT_rawudp_if,   "rawudp-if",   TYPE_ID_STR, 0, 0, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_LINUX_UDP_H)
@@ -294,6 +274,8 @@ static int stress_rawudp(stress_args_t *args)
 	}
 
 	rawudp_port += args->instance;
+	if (rawudp_port > MAX_PORT)
+		rawudp_port -= (MAX_PORT - MIN_PORT + 1);
 	reserved_port = stress_net_reserve_ports(rawudp_port, rawudp_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
@@ -308,8 +290,9 @@ static int stress_rawudp(stress_args_t *args)
 	if (stress_sighandler(args->name, SIGPIPE, stress_sock_sigpipe_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	parent_cpu = stress_get_cpu();
 	pid = fork();
@@ -340,7 +323,7 @@ finish:
 stressor_info_t stress_rawudp_info = {
 	.stressor = stress_rawudp,
 	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.supported = stress_rawudp_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -349,7 +332,7 @@ stressor_info_t stress_rawudp_info = {
 stressor_info_t stress_rawudp_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.supported = stress_rawudp_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
