@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King.
+ * Copyright (C) 2021-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-pragma.h"
 
 #define MIN_READAHEAD_BYTES	(1 * MB)
 #define MAX_READAHEAD_BYTES	(MAX_FILE_LIMIT)
@@ -34,19 +35,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_readahead_bytes(const char *opt)
-{
-	uint64_t readahead_bytes;
-
-	readahead_bytes = stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("readahead-bytes", readahead_bytes,
-		MIN_READAHEAD_BYTES, MAX_READAHEAD_BYTES);
-	return stress_set_setting("readahead-bytes", TYPE_ID_UINT64, &readahead_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_readahead_bytes,	stress_set_readahead_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_readahead_bytes, "readahead-bytes", TYPE_ID_UINT64_BYTES, MIN_READAHEAD_BYTES, MAX_READAHEAD_BYTES, NULL },
+	END_OPT,
 };
 
 #if defined(__linux__) &&	\
@@ -73,7 +64,7 @@ static void OPTIMIZE3 stress_readahead_modify_offsets(off_t *offsets)
 }
 
 static int do_readahead(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	off_t *offsets)
@@ -94,7 +85,7 @@ static int do_readahead(
  *  stress_readahead
  *	stress file system cache via readahead calls
  */
-static int stress_readahead(const stress_args_t *args)
+static int stress_readahead(stress_args_t *args)
 {
 	buffer_t *buf = NULL;
 	uint64_t rounded_readahead_bytes, i;
@@ -184,6 +175,7 @@ seq_wr_retry:
 			goto close_finish;
 		}
 
+PRAGMA_UNROLL_N(8)
 		for (j = 0; j < (BUF_SIZE / sizeof(*buf)); j++)
 			buf[j] = (buffer_t)o + j;
 
@@ -201,7 +193,7 @@ seq_wr_retry:
 		}
 	}
 
-	if (fstat(fd, &statbuf) < 0) {
+	if (shim_fstat(fd, &statbuf) < 0) {
 		pr_fail("%s: fstat failed, errno=%d (%s)%s\n",
 			args->name, errno, strerror(errno), fs_type);
 		goto close_finish;
@@ -211,6 +203,8 @@ seq_wr_retry:
 	rounded_readahead_bytes = (uint64_t)statbuf.st_size -
 		(uint64_t)(statbuf.st_size % BUF_SIZE);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	stress_readahead_generate_offsets(offsets, rounded_readahead_bytes);
@@ -243,6 +237,7 @@ rnd_rd_retry:
 				size_t j;
 				const off_t o = offsets[i] / BUF_SIZE;
 
+PRAGMA_UNROLL_N(8)
 				for (j = 0; j < (BUF_SIZE / sizeof(*buf)); j++) {
 					const buffer_t v = (buffer_t)o + j;
 
@@ -310,7 +305,7 @@ finish:
 stressor_info_t stress_readahead_info = {
 	.stressor = stress_readahead,
 	.class = CLASS_IO | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
@@ -318,7 +313,7 @@ stressor_info_t stress_readahead_info = {
 stressor_info_t stress_readahead_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
