@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2023 Colin Ian King
+ * Copyright (C) 2021-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 
+#include <sched.h>
+
 static const stress_help_t help[] = {
 	{ NULL,	"resched N",		"start N workers that spawn renicing child processes" },
 	{ NULL,	"resched-ops N",	"stop after N nice bogo nice'd yield operations" },
@@ -42,8 +44,8 @@ static void MLOCKED_TEXT stress_resched_usr1_handler(int sig)
 		stress_continue_set_flag(false);
 }
 
-static void NORETURN stress_resched_child(
-	const stress_args_t *args,
+static void OPTIMIZE3 NORETURN stress_resched_child(
+	stress_args_t *args,
 	const int niceness,
 	const int max_niceness,
 	uint64_t *yields)
@@ -75,19 +77,16 @@ static void NORETURN stress_resched_child(
 
 	for (i = niceness; i < max_niceness; i++) {
 		int k;
+		register uint64_t *const yield_ptr = &yields[i];
 
 		for (k = 0; k < 1024; k++) {
 #if defined(HAVE_SCHEDULING) && 	\
     defined(HAVE_SCHED_SETSCHEDULER)
+			struct sched_param param;
 			size_t j;
 
+			(void)shim_memset(&param, 0, sizeof(param));
 			for (j = 0; j < SIZEOF_ARRAY(normal_policies); j++) {
-				struct sched_param param;
-
-				if (!stress_continue(args))
-					break;
-
-				(void)shim_memset(&param, 0, sizeof(param));
 				param.sched_priority = 0;
 				if (sched_setscheduler(pid, normal_policies[j], &param) == 0) {
 					int ret;
@@ -103,15 +102,13 @@ static void NORETURN stress_resched_child(
 					}
 				}
 				VOID_RET(int, shim_sched_yield());
-				if (yields)
-					yields[i]++;
 				stress_bogo_inc(args);
+				(*yield_ptr)++;
 			}
 #else
 			VOID_RET(int, shim_sched_yield());
-			if (yields)
-				yields[i]++;
 			stress_bogo_inc(args);
+			(*yield_ptr)++;
 #endif
 		}
 
@@ -128,21 +125,21 @@ static void NORETURN stress_resched_child(
  *	start a child process that will re-nice itself
  */
 static void stress_resched_spawn(
-	const stress_args_t *args,
-	pid_t *pids,
-	const int index,
+	stress_args_t *args,
+	stress_pid_t *s_pids,
+	const int idx,
 	const int max_prio,
 	uint64_t *yields)
 {
 	pid_t pid;
 
-	pids[index] = -1;
+	s_pids[idx].pid = -1;
 
 	pid = fork();
 	if (pid == 0) {
-		stress_resched_child(args, index, max_prio, yields);
+		stress_resched_child(args, idx, max_prio, yields);
 	} else if (pid > 0) {
-		pids[index] = pid;
+		s_pids[idx].pid = pid;
 	}
 }
 
@@ -150,13 +147,13 @@ static void stress_resched_spawn(
  *  stress on sched_resched()
  *	stress system by sched_resched
  */
-static int stress_resched(const stress_args_t *args)
+static int stress_resched(stress_args_t *args)
 {
-	pid_t *pids;
+	stress_pid_t *s_pids;
 	int rc = EXIT_SUCCESS;
 
 #if defined(HAVE_SETPRIORITY)
-	int i, pids_max, max_prio = 19;
+	int i, s_pids_max, max_prio = 19;
 	size_t yields_size;
 	uint64_t *yields;
 
@@ -170,32 +167,39 @@ static int stress_resched(const stress_args_t *args)
 	}
 #endif
 #endif
-	pids_max = max_prio + 1; /* 0.. max_prio */
-	pids = calloc((size_t)pids_max, sizeof(*pids));
-	if (!pids) {
-		pr_inf_skip("%s: cannot allocate pids array, skipping stressor, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+	s_pids_max = max_prio + 1; /* 0.. max_prio */
+	s_pids = stress_s_pids_mmap((size_t)s_pids_max);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, s_pids_max);
 		return EXIT_NO_RESOURCE;
 	}
-	for (i = 0; i < pids_max; i++)
-		pids[i] = -1;
+	for (i = 0; i < s_pids_max; i++)
+		s_pids[i].pid = -1;
 
 	if (stress_sighandler(args->name, SIGUSR1, stress_resched_usr1_handler, NULL) < 0) {
 		rc = EXIT_NO_RESOURCE;
-		goto free_pids;
+		goto tidy_s_pids;
 	}
 
-	yields_size = ((sizeof(*yields) * (size_t)pids_max) + args->page_size - 1) & ~(args->page_size - 1);
-	yields = (uint64_t *)mmap(NULL, yields_size, PROT_READ | PROT_WRITE,
+	yields_size = ((sizeof(*yields) * (size_t)s_pids_max) + args->page_size - 1) & ~(args->page_size - 1);
+	yields = (uint64_t *)stress_mmap_populate(NULL, yields_size,
+				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_SHARED, -1, 0);
-	if (yields == MAP_FAILED)
-		yields = NULL;
+	if (yields == MAP_FAILED) {
+		pr_inf_skip("%s: cannot mmap yield counter array, skipping stressor, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		rc = EXIT_NO_RESOURCE;
+		goto tidy_s_pids;
+	}
+	stress_set_vma_anon_name(yields, yields_size, "yield-stats");
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	/* Start off one child process per positive nice level */
-	for (i = 0; stress_continue(args) && (i < pids_max); i++)
-		stress_resched_spawn(args, pids, i, max_prio, yields);
+	for (i = 0; stress_continue(args) && (i < s_pids_max); i++)
+		stress_resched_spawn(args, s_pids, i, max_prio, yields);
 
 	do {
 		pid_t pid;
@@ -211,30 +215,30 @@ static int stress_resched(const stress_args_t *args)
 			 *  Find unstarted process or process that just terminated
 			 *  and respawn it at the nice level of the given slot
 			 */
-			for (i = 0; i < pids_max; i++) {
-				if ((pids[i] == -1) || (pid == pids[i]))
-					stress_resched_spawn(args, pids, i, max_prio, yields);
+			for (i = 0; i < s_pids_max; i++) {
+				if ((s_pids[i].pid == -1) || (pid == s_pids[i].pid))
+					stress_resched_spawn(args, s_pids, i, max_prio, yields);
 			}
 		}
 	} while (stress_continue(args));
 
-	if (stress_kill_and_wait_many(args, pids, pids_max, SIGALRM, true) == EXIT_FAILURE)
+	if (stress_kill_and_wait_many(args, s_pids, s_pids_max, SIGALRM, true) == EXIT_FAILURE)
 		rc = EXIT_FAILURE;
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	/*
 	 *  Dump stats for just instance 0 to reduce output
 	 */
-	if ((yields != NULL) && (args->instance == 0)) {
+	if (args->instance == 0) {
 		uint64_t total_yields = 0;
 
-		for (i = 0; i < pids_max; i++)
+		for (i = 0; i < s_pids_max; i++)
 			total_yields += yields[i];
 
-		pr_lock();
-		for (i = 0; i < pids_max; i++) {
+		pr_block_begin();
+		for (i = 0; i < s_pids_max; i++) {
 			if (yields[i] > 0) {
-				double percent = 100.0 * ((double)yields[i] / (double)total_yields);
+				double percent = 100.0 * ((double)yields[i]/ (double)total_yields);
 
 				if (i == 0) {
 					pr_dbg("%s: prio %2d: %5.2f%% yields\n",
@@ -248,13 +252,12 @@ static int stress_resched(const stress_args_t *args)
 				}
 			}
 		}
-		pr_unlock();
+		pr_block_end();
 	}
 
-	if (yields != NULL)
-		(void)munmap((void *)yields, yields_size);
-free_pids:
-	free(pids);
+	(void)munmap((void *)yields, yields_size);
+tidy_s_pids:
+	(void)stress_s_pids_munmap(s_pids, (size_t)s_pids_max);
 
 	return rc;
 }
