@@ -183,6 +183,8 @@ static int stress_session(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	while (stress_continue(args)) {
@@ -194,6 +196,8 @@ static int stress_session(stress_args_t *args)
 				continue;
 			pr_inf("%s: fork failed: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			(void)close(fds[0]);
+			(void)close(fds[1]);
 			return EXIT_NO_RESOURCE;
 		} else if (pid == 0) {
 			/* Child */
