@@ -147,6 +147,8 @@ static int stress_set(stress_args_t *args)
 		(void)shim_strscpy(longname, hostname, longname_len);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
