@@ -127,36 +127,11 @@ static const int shm_flags[] = {
 };
 #endif
 
-static int stress_set_shm_sysv_mlock(const char *opt)
-{
-	return stress_set_setting_true("shm-sysv-mlock", opt);
-}
-
-static int stress_set_shm_sysv_bytes(const char *opt)
-{
-	size_t shm_sysv_bytes;
-
-	shm_sysv_bytes = (size_t)stress_get_uint64_byte(opt);
-	stress_check_range_bytes("shm-sysv-bytes", shm_sysv_bytes,
-		MIN_SHM_SYSV_BYTES, MAX_MEM_LIMIT);
-	return stress_set_setting("shm-sysv-bytes", TYPE_ID_SIZE_T, &shm_sysv_bytes);
-}
-
-static int stress_set_shm_sysv_segments(const char *opt)
-{
-	size_t shm_sysv_segments;
-
-	shm_sysv_segments = (size_t)stress_get_uint64(opt);
-	stress_check_range("shm-sysv-segs", shm_sysv_segments,
-		MIN_SHM_SYSV_SEGMENTS, MAX_SHM_SYSV_SEGMENTS);
-	return stress_set_setting("shm-sysv-segs", TYPE_ID_SIZE_T, &shm_sysv_segments);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_shm_sysv_bytes,		stress_set_shm_sysv_bytes },
-	{ OPT_shm_sysv_mlock,		stress_set_shm_sysv_mlock },
-	{ OPT_shm_sysv_segments,	stress_set_shm_sysv_segments },
-	{ 0,				NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_shm_sysv_bytes, "shm-sysv-bytes", TYPE_ID_SIZE_T_BYTES, MIN_SHM_SYSV_BYTES, MAX_MEM_LIMIT, NULL },
+	{ OPT_shm_sysv_mlock, "shm-sysv-mlock", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_shm_sysv_segs,  "shm-sysv-segs",  TYPE_ID_SIZE_T, MIN_SHM_SYSV_SEGMENTS, MAX_SHM_SYSV_SEGMENTS, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SHM_SYSV)
@@ -165,13 +140,13 @@ static void stress_shm_metrics(
 	stress_args_t *args,
 	const double duration,
 	const double count,
-	const char *syscall,
+	const char *syscall_name,
 	const int idx)
 {
 	char buffer[40];
 	const double rate = (count > 0.0) ? (duration / count) : 0.0;
 
-	(void)snprintf(buffer, sizeof(buffer), "nanosecs per %s call", syscall);
+	(void)snprintf(buffer, sizeof(buffer), "nanosecs per %s call", syscall_name);
 	stress_metrics_set(args, idx, buffer,
 		STRESS_DBL_NANOSECOND * rate, STRESS_METRIC_HARMONIC_MEAN);
 }
@@ -982,8 +957,9 @@ static int stress_shm_sysv(stress_args_t *args)
 
 	orig_sz = sz = shm_sysv_bytes & ~(page_size - 1);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	while (stress_continue_flag() && retry) {
 		if (pipe(pipefds) < 0) {
@@ -1109,16 +1085,16 @@ fork_again:
 
 stressor_info_t stress_shm_sysv_info = {
 	.stressor = stress_shm_sysv,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_VM | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_shm_sysv_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_VM | CLASS_OS | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without System V shared memory shmat() shmdt() system calls"
