@@ -279,6 +279,7 @@ static int stress_sigsegv(stress_args_t *args)
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(ro_ptr, args->page_size, "ro-page");
 	/* Allocate write only page */
 	none_ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_NONE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
@@ -289,7 +290,10 @@ static int stress_sigsegv(stress_args_t *args)
 		(void)munmap((void *)ro_ptr, args->page_size);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(ro_ptr, args->page_size, "no-page");
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	mask = 0;
@@ -355,11 +359,14 @@ static int stress_sigsegv(stress_args_t *args)
 				pr_fail("%s: expecting SIGSEGV/SIGILL/SIGBUS, got %s instead\n",
 					args->name, strsignal(signo));
 			}
-#if defined(SEGV_ACCERR) && 	\
-    !defined(__OpenBSD__)
-			if (verify && (signo == SIGBUS) && (code != SEGV_ACCERR)) {
-				pr_fail("%s: expecting SIGBUS si_code SEGV_ACCERR (%d), got %d instead\n",
-					args->name, SEGV_ACCERR, code);
+#if defined(BUS_OBJERR) &&	\
+    defined(BUS_ADRERR)
+			if (verify && (signo == SIGBUS) &&
+				      (code != BUS_OBJERR) &&
+				      (code != BUS_ADRERR)) {
+				pr_fail("%s: expecting SIGBUS si_code BUS_OBJERR (%d) "
+					"or BUS_ADRERR (%d), got %d instead\n",
+					args->name, BUS_OBJERR, BUS_ADRERR, code);
 			}
 #endif
 #endif
