@@ -56,6 +56,8 @@ static int stress_sigtrap(stress_args_t *args)
 	if (stress_sighandler(args->name, SIGTRAP, stress_sigtrap_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	while (stress_continue(args)) {
@@ -89,14 +91,14 @@ static int stress_sigtrap(stress_args_t *args)
 	}
 	rate = (counter > 0) ? duration / (double)counter : 0.0;
 	stress_metrics_set(args, 0, "nanosecs to handle SIGTRAP",
-		rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 
 	return EXIT_SUCCESS;
 }
 
 stressor_info_t stress_sigtrap_info = {
 	.stressor = stress_sigtrap,
-	.class = CLASS_INTERRUPT | CLASS_OS,
+	.class = CLASS_SIGNAL | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
