@@ -41,20 +41,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		  NULL }
 };
 
-/*
- *  stress_set_skiplist_size()
- *	set skiplist size from given option string
- */
-static int stress_set_skiplist_size(const char *opt)
-{
-	uint64_t skiplist_size;
-
-	skiplist_size = stress_get_uint64(opt);
-	stress_check_range("skiplist-size", skiplist_size,
-		MIN_SKIPLIST_SIZE, MAX_SKIPLIST_SIZE);
-	return stress_set_setting("skiplist-size", TYPE_ID_UINT64, &skiplist_size);
-}
-
 /*
  *  skip_list_random_level()
  *	generate a quasi-random skip list level
@@ -244,6 +230,8 @@ static int OPTIMIZE3 stress_skiplist(stress_args_t *args)
 		goto finish;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -269,9 +257,13 @@ static int OPTIMIZE3 stress_skiplist(stress_args_t *args)
 		for (i = 0; i < n; i++) {
 			const unsigned long v = (i >> 1) ^ i;
 
-			if (!skip_list_search(&list, v))
+			if (!skip_list_search(&list, v)) {
 				pr_fail("%s node containing value %lu was not found\n",
 					args->name, v);
+				rc = EXIT_FAILURE;
+				skip_list_free(&list);
+				goto finish;
+			}
 		}
 		skip_list_free(&list);
 
@@ -285,15 +277,15 @@ finish:
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_skiplist_size,	stress_set_skiplist_size },
-	{ 0,			NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_skiplist_size, "skiplist-size", TYPE_ID_UINT64, MIN_SKIPLIST_SIZE, MAX_SKIPLIST_SIZE, NULL },
+	END_OPT,
 };
 
 stressor_info_t stress_skiplist_info = {
 	.stressor = stress_skiplist,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
