@@ -625,30 +625,6 @@ static size_t stress_str_all(stress_args_t *args, stress_str_args_t *info)
 	return 0;
 }
 
-/*
- *  stress_set_str_method()
- *	set the default string stress method
- */
-static int stress_set_str_method(const char *name)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(str_methods); i++) {
-		if (!strcmp(str_methods[i].name, name)) {
-			stress_set_setting("str-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "str-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(str_methods); i++) {
-		(void)fprintf(stderr, " %s", str_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
-
 /*
  *  stress_str()
  *	stress CPU by doing various string operations
@@ -681,6 +657,8 @@ static int stress_str(stress_args_t *args)
 		metrics[i].count = 0.0;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -712,8 +690,8 @@ static int stress_str(stress_args_t *args)
 			const double rate = metrics[i].count / metrics[i].duration;
 
 			(void)snprintf(msg, sizeof(msg), "%s calls per sec", str_methods[i].name);
-			stress_metrics_set(args, j, msg,	
-				rate, STRESS_HARMONIC_MEAN);
+			stress_metrics_set(args, j, msg,
+				rate, STRESS_METRIC_HARMONIC_MEAN);
 			j++;
 		}
 	}
@@ -721,21 +699,20 @@ static int stress_str(stress_args_t *args)
 	return info.failed ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
-static void stress_str_set_default(void)
+static const char *stress_str_method(const size_t i)
 {
-	stress_set_str_method("all");
+	return (i < SIZEOF_ARRAY(str_methods)) ? str_methods[i].name : NULL;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_str_method,	stress_set_str_method },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_str_method, "str-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_str_method },
+	END_OPT,
 };
 
 stressor_info_t stress_str_info = {
 	.stressor = stress_str,
-	.set_default = stress_str_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
 	.verify = VERIFY_OPTIONAL,
+	.opts = opts,
 	.help = help
 };
