@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,19 +43,9 @@ static const unsigned int sync_modes[] = {
 UNEXPECTED
 #endif
 
-static int stress_set_sync_file_bytes(const char *opt)
-{
-	off_t sync_file_bytes;
-
-	sync_file_bytes = (off_t)stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("sync_file-bytes", (uint64_t)sync_file_bytes,
-		MIN_SYNC_FILE_BYTES, MAX_SYNC_FILE_BYTES);
-	return stress_set_setting("sync_file-bytes", TYPE_ID_OFF_T, &sync_file_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_sync_file_bytes,	stress_set_sync_file_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_sync_file_bytes, "sync-file-bytes", TYPE_ID_OFF_T, MIN_SYNC_FILE_BYTES, MAX_SYNC_FILE_BYTES, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SYNC_FILE_RANGE)
@@ -65,7 +55,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *
  */
 static int stress_sync_allocate(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const char *fs_type,
 	const off_t sync_file_bytes)
@@ -109,9 +99,9 @@ static int stress_sync_allocate(
  *  stress_sync_file
  *	stress the sync_file_range system call
  */
-static int stress_sync_file(const stress_args_t *args)
+static int stress_sync_file(stress_args_t *args)
 {
-	int fd, ret;
+	int fd, ret, rc = EXIT_SUCCESS;
 	const int bad_fd = stress_get_bad_fd();
 	off_t sync_file_bytes = DEFAULT_SYNC_FILE_BYTES;
 	char filename[PATH_MAX];
@@ -146,9 +136,21 @@ static int stress_sync_file(const stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
 	}
-	fs_type = stress_fs_type(filename);
+	stress_file_rw_hint_short(fd);
+
+	fs_type = stress_get_fs_type(filename);
+#if defined(HAVE_PATHCONF)
+#if defined(_PC_ASYNC_IO)
+	VOID_RET(long, pathconf(filename, _PC_ASYNC_IO));
+#endif
+#if defined(_PC_SYNC_IO)
+	VOID_RET(long, pathconf(filename, _PC_SYNC_IO));
+#endif
+#endif
 	(void)shim_unlink(filename);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -162,7 +164,7 @@ static int stress_sync_file(const stress_args_t *args)
 				continue;
 			break;
 		}
-		for (offset = 0; keep_stressing_flag() &&
+		for (offset = 0; stress_continue_flag() &&
 		     (offset < (shim_off64_t)sync_file_bytes); ) {
 			const shim_off64_t sz = (stress_mwc32() & 0x1fc00) + KB;
 
@@ -175,11 +177,12 @@ static int stress_sync_file(const stress_args_t *args)
 				}
 				pr_fail("%s: sync_file_range (forward), errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
+				rc = EXIT_FAILURE;
 				break;
 			}
 			offset += sz;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		/*
@@ -202,7 +205,7 @@ static int stress_sync_file(const stress_args_t *args)
 				continue;
 			break;
 		}
-		for (offset = 0; keep_stressing_flag() &&
+		for (offset = 0; stress_continue_flag() &&
 		     (offset < (shim_off64_t)sync_file_bytes); ) {
 			const shim_off64_t sz = (stress_mwc32() & 0x1fc00) + KB;
 
@@ -215,11 +218,12 @@ static int stress_sync_file(const stress_args_t *args)
 				}
 				pr_fail("%s: sync_file_range (reverse), errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
+				rc = EXIT_FAILURE;
 				break;
 			}
 			offset += sz;
 		}
-		if (!keep_stressing_flag())
+		if (!stress_continue_flag())
 			break;
 
 		ret = stress_sync_allocate(args, fd, fs_type, sync_file_bytes);
@@ -228,8 +232,8 @@ static int stress_sync_file(const stress_args_t *args)
 				continue;
 			break;
 		}
-		for (i = 0; keep_stressing_flag() && i <
-		     (shim_off64_t)sync_file_bytes / (shim_off64_t)(128 * KB); i++) {
+		for (i = 0; stress_continue_flag() &&
+		     (i < (shim_off64_t)sync_file_bytes / (shim_off64_t)(128 * KB)); i++) {
 			offset = (shim_off64_t)(stress_mwc64modn((uint64_t)sync_file_bytes) & ~((128 * KB) - 1));
 
 			ret = shim_sync_file_range(fd, offset, 128 * KB, mode);
@@ -241,24 +245,25 @@ static int stress_sync_file(const stress_args_t *args)
 				}
 				pr_fail("%s: sync_file_range (random), errno=%d (%s)%s\n",
 					args->name, errno, strerror(errno), fs_type);
+				rc = EXIT_FAILURE;
 				break;
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fd);
 	(void)stress_temp_dir_rm_args(args);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_sync_file_info = {
 	.stressor = stress_sync_file,
 	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -266,7 +271,8 @@ stressor_info_t stress_sync_file_info = {
 stressor_info_t stress_sync_file_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sync_file_range() system call"
 };
