@@ -139,14 +139,14 @@ static void stress_tee_pipe_read(stress_args_t *args, int fds[2])
 			if (UNLIKELY(ret < 0)) {
 				switch (errno) {
 				case EPIPE:
-					goto finish;
+					return;
 				case EAGAIN:
 				case EINTR:
 					continue;
 				default:
 					pr_fail("%s: unexpected read error, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-					goto finish;
+					return;
 				}
 			} else {
 				n += (size_t)ret;
@@ -162,8 +162,6 @@ static void stress_tee_pipe_read(stress_args_t *args, int fds[2])
 		}
 		counter++;
 	}
-finish:
-	(void)close(fds[1]);
 }
 
 /*
@@ -244,17 +242,17 @@ static int stress_tee(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	pids[0] = stress_tee_spawn(args, stress_tee_pipe_write, pipe_in);
-	if (pids[0] < 0) {
-		(void)close(fd);
-		return EXIT_FAILURE;
-	}
+	if (pids[0] < 0)
+		goto tidy_fd;
 	(void)close(pipe_in[1]);
 
 	pids[1] = stress_tee_spawn(args, stress_tee_pipe_read, pipe_out);
-	if (pids[0] < 0)
+	if (pids[1] < 0)
 		goto tidy_child1;
 	(void)close(pipe_out[0]);
 
@@ -323,7 +321,7 @@ do_splice:
 
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
 	stress_metrics_set(args, 0, "MB per sec tee rate",
-		rate / (double)MB, STRESS_HARMONIC_MEAN);
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
 
 tidy_child2:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -334,7 +332,7 @@ tidy_child1:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(pipe_in[0]);
 	(void)stress_kill_pid_wait(pids[0], NULL);
-
+tidy_fd:
 	(void)close(fd);
 
 	return ret;
