@@ -20,6 +20,8 @@
 #include "stress-ng.h"
 #include "core-capabilities.h"
 
+#include <sys/ioctl.h>
+
 #if defined(HAVE_LINUX_RANDOM_H)
 #include <linux/random.h>
 #else
@@ -40,7 +42,10 @@ static const stress_help_t help[] = {
 static void check_eperm(stress_args_t *args, const ssize_t ret, const int err)
 {
 	if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-	    ((ret == 0) || ((err != EPERM) && (err != EINVAL) && (err != ENOTTY)))) {
+	    ((ret == 0) || ((err != EPERM) &&
+			    (err != ENOSYS) &&
+			    (err != EINVAL) &&
+			    (err != ENOTTY)))) {
 		pr_fail("%s: expected errno to be EPERM, got errno %d (%s) instead\n",
 			args->name, err, strerror(err));
 	}
@@ -113,6 +118,8 @@ static int stress_urandom(stress_args_t *args)
 		return EXIT_NOT_IMPLEMENTED;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -315,10 +322,10 @@ next:
 	} while (stress_continue(args));
 
 	stress_metrics_set(args, 0, "million random bits read",
-		bytes * 8.0 / 1000000.0, STRESS_GEOMETRIC_MEAN);
+		bytes * 8.0 / 1000000.0, STRESS_METRIC_GEOMETRIC_MEAN);
 	rate = (duration > 0.0) ? bytes * 8.0 / duration : 0.0;
 	stress_metrics_set(args, 1, "million random bits per sec",
-		rate / 1000000.0, STRESS_HARMONIC_MEAN);
+		rate / 1000000.0, STRESS_METRIC_HARMONIC_MEAN);
 
 	rc = EXIT_SUCCESS;
 err:
