@@ -36,14 +36,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-static int stress_set_utime_fsync(const char *opt)
-{
-	return stress_set_setting_true("utime-fsync", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_utime_fsync,	stress_set_utime_fsync },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_utime_fsync, "utime-fsync", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_UTIME_H)
@@ -73,6 +68,7 @@ static int OPTIMIZE3 stress_utime(stress_args_t *args)
 	char path[PATH_MAX];
 #endif
 	int dir_fd = -1;
+	int rc = EXIT_SUCCESS;
 	char filename[PATH_MAX];
 	char hugename[PATH_MAX + 16];
 	int ret, fd;
@@ -112,8 +108,14 @@ static int OPTIMIZE3 stress_utime(stress_args_t *args)
 		return ret;
 	}
 
+#if defined(HAVE_PATHCONF) &&	\
+    defined(_PC_TIMESTAMP_RESOLUTION)
+	VOID_RET(long, pathconf(filename, _PC_TIMESTAMP_RESOLUTION));
+#endif
 	stress_rndstr(hugename, sizeof(hugename));
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -392,6 +394,7 @@ STRESS_PRAGMA_POP
 		{
 			struct utimbuf utbuf;
 			struct timeval tv;
+			int i;
 
 			(void)gettimeofday(&tv, NULL);
 			utbuf.actime = (time_t)tv.tv_sec;
@@ -402,6 +405,7 @@ STRESS_PRAGMA_POP
 					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno),
 						stress_get_fs_type(filename));
+					rc = EXIT_FAILURE;
 					break;
 				}
 			} else {
@@ -410,6 +414,7 @@ STRESS_PRAGMA_POP
 					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno),
 						stress_get_fs_type(filename));
+					rc = EXIT_FAILURE;
 					break;
 				}
 				duration += stress_time_now() - t;
@@ -423,10 +428,14 @@ STRESS_PRAGMA_POP
 					if (statbuf.st_atime < tv.tv_sec) {
 						pr_fail("%s: utime failed: access time is less than expected time\n",
 							args->name);
+						rc = EXIT_FAILURE;
+						break;
 					}
 					if (statbuf.st_mtime < tv.tv_sec) {
 						pr_fail("%s: utime failed: modified time is less than expected time\n",
 							args->name);
+						rc = EXIT_FAILURE;
+						break;
 					}
 				}
 			}
@@ -435,6 +444,7 @@ STRESS_PRAGMA_POP
 					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno),
 						stress_get_fs_type(filename));
+					rc = EXIT_FAILURE;
 					break;
 				}
 			} else {
@@ -443,6 +453,7 @@ STRESS_PRAGMA_POP
 					pr_fail("%s: utime failed: errno=%d (%s)%s\n",
 						args->name, errno, strerror(errno),
 						stress_get_fs_type(filename));
+					rc = EXIT_FAILURE;
 					break;
 				}
 				duration += stress_time_now() - t;
@@ -454,6 +465,20 @@ STRESS_PRAGMA_POP
 
 			/* Exercise huge filename, ENAMETOOLONG */
 			VOID_RET(int, shim_utime(hugename, &utbuf));
+
+			/* Exercise ranges of +ve times */
+			utbuf.actime = (1ULL << ((sizeof(time_t) * 8) - 1)) - 1;
+			utbuf.modtime = utbuf.actime;
+			shim_utime(filename, &utbuf);
+			for (i = 0; utbuf.actime && (i < 64); i++) {
+				utbuf.actime >>= 1;
+				utbuf.modtime = utbuf.actime;
+				shim_utime(filename, &utbuf);
+			}
+
+			utbuf.actime = ~(time_t)0;
+			utbuf.modtime = utbuf.actime;
+			shim_utime(filename, &utbuf);
 		}
 		/* forces metadata writeback */
 		if (utime_fsync) {
@@ -471,7 +496,7 @@ STRESS_PRAGMA_POP
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
 	stress_metrics_set(args, 0, "utime calls per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 #if defined(O_DIRECTORY) &&	\
     defined(O_PATH) &&		\
@@ -483,13 +508,13 @@ STRESS_PRAGMA_POP
 	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 stressor_info_t stress_utime_info = {
 	.stressor = stress_utime,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
@@ -499,7 +524,7 @@ stressor_info_t stress_utime_info = {
 stressor_info_t stress_utime_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without utime.h"
