@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2024 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 
+#include <sys/ioctl.h>
+
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
 #else
@@ -69,7 +71,7 @@ struct shim_fsverity_read_metadata_arg {
  *  stress_verity
  *	stress file verity
  */
-static int stress_verity(const stress_args_t *args)
+static int stress_verity(stress_args_t *args)
 {
 	char filename[PATH_MAX];
 	int ret, fd;
@@ -88,6 +90,8 @@ static int stress_verity(const stress_args_t *args)
 
 	(void)stress_temp_filename_args(args, filename, sizeof(filename), stress_mwc32());
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
