@@ -88,7 +88,10 @@ static const stress_help_t help[] = {
 };
 
 static const stress_vm_madvise_info_t vm_madvise_info[] = {
-#if defined(HAVE_MADVISE)
+#if !defined(HAVE_MADVISE)
+	/* No MADVISE, default to normal, ignored */
+	{ "normal",	0 },
+#else
 #if defined(MADV_DONTNEED)
 	{ "dontneed",	MADV_DONTNEED},
 #endif
@@ -116,10 +119,6 @@ static const stress_vm_madvise_info_t vm_madvise_info[] = {
 #if defined(MADV_WILLNEED)
 	{ "willneed",	MADV_WILLNEED},
 #endif
-	{ NULL,         0 },
-#else
-	/* No MADVISE, default to normal, ignored */
-	{ "normal",	0 },
 #endif
 };
 
@@ -127,88 +126,12 @@ static const stress_vm_madvise_info_t vm_madvise_info[] = {
  *  stress_continue(args)
  *	returns true if we can keep on running a stressor
  */
-static bool HOT OPTIMIZE3 stress_continue_vm(stress_args_t *args)
+static bool OPTIMIZE3 stress_continue_vm(stress_args_t *args)
 {
 	return (LIKELY(stress_continue_flag()) &&
 	        LIKELY(!args->max_ops || ((stress_bogo_get(args) >> VM_BOGO_SHIFT) < args->max_ops)));
 }
 
-static int stress_set_vm_hang(const char *opt)
-{
-	uint64_t vm_hang;
-
-	vm_hang = stress_get_uint64_time(opt);
-	stress_check_range("vm-hang", vm_hang,
-		MIN_VM_HANG, MAX_VM_HANG);
-	return stress_set_setting("vm-hang", TYPE_ID_UINT64, &vm_hang);
-}
-
-static int stress_set_vm_bytes(const char *opt)
-{
-	size_t vm_bytes;
-
-	vm_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("vm-bytes", vm_bytes,
-		MIN_VM_BYTES, MAX_VM_BYTES);
-	return stress_set_setting("vm-bytes", TYPE_ID_SIZE_T, &vm_bytes);
-}
-
-#if defined(MAP_LOCKED) || defined(MAP_POPULATE)
-static int stress_set_vm_flags(const int flag)
-{
-	int vm_flags = 0;
-
-	(void)stress_get_setting("vm-flags", &vm_flags);
-	vm_flags |= flag;
-	return stress_set_setting("vm-flags", TYPE_ID_INT, &vm_flags);
-}
-#endif
-
-static int stress_set_vm_mmap_locked(const char *opt)
-{
-	(void)opt;
-
-#if defined(MAP_LOCKED)
-	return stress_set_vm_flags(MAP_LOCKED);
-#else
-	return 0;
-#endif
-}
-
-static int stress_set_vm_mmap_populate(const char *opt)
-{
-	(void)opt;
-
-#if defined(MAP_POPULATE)
-	return stress_set_vm_flags(MAP_POPULATE);
-#else
-	return 0;
-#endif
-}
-
-static int stress_set_vm_madvise(const char *opt)
-{
-	const stress_vm_madvise_info_t *info;
-
-	for (info = vm_madvise_info; info->name; info++) {
-		if (!strcmp(opt, info->name)) {
-			stress_set_setting("vm-madvise", TYPE_ID_INT, &info->advice);
-			return 0;
-		}
-	}
-	(void)fprintf(stderr, "invalid vm-madvise advice '%s', allowed advice options are:", opt);
-	for (info = vm_madvise_info; info->name; info++) {
-		(void)fprintf(stderr, " %s", info->name);
-	}
-	(void)fprintf(stderr, "\n");
-	return -1;
-}
-
-static int stress_set_vm_keep(const char *opt)
-{
-	return stress_set_setting_true("vm-keep", opt);
-}
-
 #define SET_AND_TEST(ptr, val, bit_errors)	\
 do {						\
 	*ptr = val;				\
@@ -280,7 +203,9 @@ static inline void inject_random_bit_errors(uint8_t *buf, const size_t sz)
  */
 static inline ALWAYS_INLINE PURE OPTIMIZE3 uint64_t stress_vm_mod(register uint64_t a, register const size_t b)
 {
-	while (LIKELY(a >= b))
+	if (LIKELY(a >= b))
+		a -= b;
+	if (UNLIKELY(a >= b))
 		a -= b;
 	return a;
 }
@@ -333,19 +258,17 @@ static inline size_t PURE stress_vm_count_bits(uint64_t v)
 	if (sizeof(unsigned long) == sizeof(uint64_t)) {
 		return (size_t)__builtin_popcountl((unsigned long)v);
 	} else if (sizeof(unsigned long) == sizeof(uint32_t)) {
-		unsigned long lo, hi;
+		const unsigned long lo = (unsigned long)(v >> 32);
+		const unsigned long hi = (unsigned long)(v & 0xffffffff);
 
-		lo = (unsigned long)(v >> 32);
-		hi = (unsigned long)(v & 0xffffffff);
 		return (size_t)__builtin_popcountl(hi) + __builtin_popcountl(lo);
 	}
 #endif
 #if defined(HAVE_BUILTIN_POPCOUNT)
 	if (sizeof(unsigned int) == sizeof(uint32_t)) {
-		unsigned int lo, hi;
+		const unsigned int lo = (unsigned int)(v >> 32);
+		const unsigned int hi = (unsigned int)(v & 0xffffffff);
 
-		lo = (unsigned int)(v >> 32);
-		hi = (unsigned int)(v & 0xffffffff);
 		return (size_t)__builtin_popcount(hi) + __builtin_popcount(lo);
 	}
 #else
@@ -721,6 +644,79 @@ static size_t TARGET_CLONES stress_vm_walking_zero_addr(
 	return bit_errors;
 }
 
+/*
+ *  stress_vm_walking_flush_data()
+ *	for each byte, walk through each byte flushing data
+ */
+static size_t TARGET_CLONES stress_vm_walking_flush_data(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	stress_args_t *args,
+	const uint64_t max_ops)
+{
+	size_t bit_errors = 0;
+	register uint8_t *ptr;
+	register uint64_t c = stress_bogo_get(args);
+	register uint8_t val = 0;
+
+	(void)sz;
+
+	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end - 7; ptr++, val++) {
+		*(ptr + 0) = (val + 0) & 0xff;
+		shim_clflush(ptr + 0);
+		stress_asm_mb();
+		*(ptr + 1) = (val + 1) & 0xff;
+		shim_clflush(ptr + 1);
+		stress_asm_mb();
+		*(ptr + 2) = (val + 2) & 0xff;
+		shim_clflush(ptr + 2);
+		stress_asm_mb();
+		*(ptr + 3) = (val + 3) & 0xff;
+		shim_clflush(ptr + 3);
+		stress_asm_mb();
+		*(ptr + 4) = (val + 4) & 0xff;
+		shim_clflush(ptr + 4);
+		stress_asm_mb();
+		*(ptr + 5) = (val + 5) & 0xff;
+		shim_clflush(ptr + 5);
+		stress_asm_mb();
+		*(ptr + 6) = (val + 6) & 0xff;
+		shim_clflush(ptr + 6);
+		stress_asm_mb();
+		*(ptr + 7) = (val + 7) & 0xff;
+		shim_mfence();
+
+		bit_errors += (*(ptr + 0) != ((val + 0) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 1) != ((val + 1) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 2) != ((val + 2) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 3) != ((val + 3) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 4) != ((val + 4) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 5) != ((val + 5) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 6) != ((val + 6) & 0xff));
+		stress_asm_mb();
+		bit_errors += (*(ptr + 7) != ((val + 7) & 0xff));
+		stress_asm_mb();
+
+		c++;
+		if (UNLIKELY(max_ops && (c >= max_ops)))
+			break;
+		if (UNLIKELY(!stress_continue_flag()))
+			break;
+	}
+	stress_vm_check("walking flush (data)", bit_errors);
+	stress_bogo_set(args, c);
+
+	return bit_errors;
+}
+
+
 /*
  *  stress_vm_gray()
  *	fill all of memory with a gray code and check that
@@ -1754,9 +1750,9 @@ static size_t TARGET_CLONES stress_vm_prime_zero(
 	size_t bit_errors = 0;
 	register uint64_t c = stress_bogo_get(args);
 	register uint8_t i = 0;
-	register size_t prime = 61; /* prime less than cache line size */
+	register const size_t prime = 61; /* prime less than cache line size */
 	static size_t offset = 0;
-	register uint8_t *ptr = (uint8_t *)buf + offset;
+	register uint8_t *ptr;
 
 #if SIZE_MAX > UINT32_MAX
 	/* Unlikely.. */
@@ -1814,9 +1810,9 @@ static size_t TARGET_CLONES stress_vm_prime_one(
 	size_t bit_errors = 0;
 	uint64_t c = stress_bogo_get(args);
 	register uint8_t i = 0;
-	register size_t prime = 61; /* prime less than cache line size */
+	register const size_t prime = 61; /* prime less than cache line size */
 	static size_t offset = 0;
-	register uint8_t *ptr = (uint8_t *)buf + offset;
+	register uint8_t *ptr;
 
 #if SIZE_MAX > UINT32_MAX
 	/* Unlikely.. */
@@ -1849,7 +1845,7 @@ static size_t TARGET_CLONES stress_vm_prime_one(
 	for (ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += prime)
 		bit_errors += 8 - stress_vm_count_bits8(*ptr);
 
-	stress_vm_check("prime-zero", bit_errors);
+	stress_vm_check("prime-one", bit_errors);
 abort:
 	offset++;
 	if (offset >= prime)
@@ -1874,9 +1870,9 @@ static size_t TARGET_CLONES stress_vm_prime_gray_zero(
 	size_t bit_errors = 0;
 	register uint64_t c = stress_bogo_get(args);
 	register uint8_t i = 0;
-	register size_t prime = 61; /* prime less than cache line size */
+	register const size_t prime = 61; /* prime less than cache line size */
 	static size_t offset = 0;
-	register uint8_t *ptr = (uint8_t *)buf + offset;
+	register uint8_t *ptr;
 
 #if SIZE_MAX > UINT32_MAX
 	/* Unlikely.. */
@@ -1934,9 +1930,9 @@ static size_t TARGET_CLONES stress_vm_prime_gray_one(
 	size_t bit_errors = 0;
 	register uint64_t c = stress_bogo_get(args);
 	register uint8_t i = 0;
-	register size_t prime = 61; /* prime less than cache line size */
+	register const size_t prime = 61; /* prime less than cache line size */
 	static size_t offset = 0;
-	register uint8_t *ptr = (uint8_t *)buf + offset;
+	register uint8_t *ptr;
 
 #if SIZE_MAX > UINT32_MAX
 	/* Unlikely.. */
@@ -3183,6 +3179,7 @@ static const stress_vm_method_info_t vm_methods[] = {
 	{ "walk-1d",		stress_vm_walking_one_data },
 	{ "walk-0a",		stress_vm_walking_zero_addr },
 	{ "walk-1a",		stress_vm_walking_one_addr },
+	{ "walk-flush",		stress_vm_walking_flush_data },
 	{ "write64",		stress_vm_write64 },
 #if defined(HAVE_NT_STORE64)
 	{ "write64nt",		stress_vm_write64nt },
@@ -3215,29 +3212,17 @@ static size_t stress_vm_all(
 	return bit_errors;
 }
 
-/*
- *  stress_set_vm_method()
- *      set default vm stress method
- */
-static int stress_set_vm_method(const char *name)
+#if defined(MAP_LOCKED) ||	\
+    defined(MAP_POPULATE)
+static void stress_vm_flags(const char *opt, int *vm_flags, const int bitmask)
 {
-	size_t i;
+	bool flag = false;
 
-	for (i = 0; i < SIZEOF_ARRAY(vm_methods); i++) {
-		if (!strcmp(vm_methods[i].name, name)) {
-			stress_set_setting("vm-method", TYPE_ID_SIZE_T, &i);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "vm-method must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(vm_methods); i++) {
-		(void)fprintf(stderr, " %s", vm_methods[i].name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
+	(void)stress_get_setting(opt, &flag);
+	if (flag)
+		*vm_flags |= bitmask;
 }
+#endif
 
 static int stress_vm_child(stress_args_t *args, void *ctxt)
 {
@@ -3246,7 +3231,8 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 	uint64_t vm_hang = DEFAULT_VM_HANG;
 	void *buf = NULL, *buf_end = NULL;
 	int vm_flags = 0;                      /* VM mmap flags */
-	int vm_madvise = -1;
+	size_t vm_madvise = 0;
+	int advice = -1;
 	int rc = EXIT_SUCCESS;
 	size_t buf_sz;
 	size_t vm_bytes = DEFAULT_VM_BYTES;
@@ -3259,6 +3245,13 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 
 	(void)stress_get_setting("vm-hang", &vm_hang);
 	(void)stress_get_setting("vm-keep", &vm_keep);
+#if defined(MAP_LOCKED)
+	stress_vm_flags("vm-mmap-locked", &vm_flags, MAP_LOCKED);
+#endif
+#if defined(MAP_POPULATE)
+	stress_vm_flags("vm-mmap-populate", &vm_flags, MAP_POPULATE);
+#endif
+
 	(void)stress_get_setting("vm-flags", &vm_flags);
 
 	if (!stress_get_setting("vm-bytes", &vm_bytes)) {
@@ -3271,7 +3264,8 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 	if (vm_bytes < MIN_VM_BYTES)
 		vm_bytes = MIN_VM_BYTES;
 	buf_sz = vm_bytes & ~(page_size - 1);
-	(void)stress_get_setting("vm-madvise", &vm_madvise);
+	if (stress_get_setting("vm-madvise", &vm_madvise))
+		advice = vm_madvise_info[vm_madvise].advice;
 
 	do {
 		if (no_mem_retries >= NO_MEM_RETRIES_MAX) {
@@ -3286,7 +3280,17 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 			if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(buf_sz)) {
 				buf = MAP_FAILED;
 			} else {
+#if defined(HAVE_MPROTECT) &&	\
+    defined(PROT_NONE)
+				/*
+				 *   allocate buffer + one trailing page
+				 *   so the last page can be marked PROT_NONE later
+				 *   to catch any buffer over-runs.
+				 */
+				buf = (uint8_t *)mmap(NULL, buf_sz + page_size,
+#else
 				buf = (uint8_t *)mmap(NULL, buf_sz,
+#endif
 					PROT_READ | PROT_WRITE,
 					MAP_PRIVATE | MAP_ANONYMOUS |
 					vm_flags, -1, 0);
@@ -3298,10 +3302,19 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 				continue;	/* Try again */
 			}
 			buf_end = (void *)((uint8_t *)buf + buf_sz);
-			if (vm_madvise < 0)
+#if defined(HAVE_MPROTECT) &&	\
+    defined(PROT_NONE)
+			/*
+			 * page after end of buffer is not readable or writable
+			 * to catch any buffer overruns
+			 */
+			(void)mprotect(buf_end, page_size, PROT_NONE);
+#endif
+
+			if (advice < 0)
 				(void)stress_madvise_random(buf, buf_sz);
 			else
-				(void)shim_madvise(buf, buf_sz, vm_madvise);
+				(void)shim_madvise(buf, buf_sz, advice);
 		}
 
 		no_mem_retries = 0;
@@ -3318,12 +3331,23 @@ static int stress_vm_child(stress_args_t *args, void *ctxt)
 
 		if (!vm_keep) {
 			(void)stress_madvise_random(buf, buf_sz);
+#if defined(HAVE_MPROTECT) &&	\
+    defined(PROT_NONE)
+			(void)stress_munmap_retry_enomem(buf, buf_sz + page_size);
+#else
 			(void)stress_munmap_retry_enomem(buf, buf_sz);
+#endif
 		}
 	} while (stress_continue_vm(args));
 
-	if (vm_keep && (buf != NULL))
-		(void)stress_munmap_retry_enomem((void *)buf, buf_sz);
+	if (vm_keep && (buf != NULL)) {
+#if defined(HAVE_MPROTECT) && 	\
+    defined(PROT_NONE)
+		(void)stress_munmap_retry_enomem(buf, buf_sz + page_size);
+#else
+		(void)stress_munmap_retry_enomem(buf, buf_sz);
+#endif
+	}
 
 	return rc;
 }
@@ -3400,6 +3424,8 @@ static int stress_vm(stress_args_t *args)
 
 	*context.bit_error_count = 0ULL;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	ret = stress_oomable_child(args, &context, stress_vm_child, STRESS_OOMABLE_NORMAL);
@@ -3421,27 +3447,31 @@ static int stress_vm(stress_args_t *args)
 	return ret;
 }
 
-static void stress_vm_set_default(void)
+static const char *stress_vm_madvise(const size_t i)
 {
-	stress_set_vm_method("all");
+	return (i < SIZEOF_ARRAY(vm_madvise_info)) ? vm_madvise_info[i].name : NULL;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_vm_bytes,		stress_set_vm_bytes },
-	{ OPT_vm_hang,		stress_set_vm_hang },
-	{ OPT_vm_keep,		stress_set_vm_keep },
-	{ OPT_vm_madvise,	stress_set_vm_madvise },
-	{ OPT_vm_method,	stress_set_vm_method },
-	{ OPT_vm_mmap_locked,	stress_set_vm_mmap_locked },
-	{ OPT_vm_mmap_populate,	stress_set_vm_mmap_populate },
-	{ 0,			NULL }
+static const char *stress_vm_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(vm_methods)) ? vm_methods[i].name : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_vm_bytes,    "vm-bytes",    TYPE_ID_SIZE_T_BYTES, MIN_VM_BYTES, MAX_VM_BYTES, NULL },
+	{ OPT_vm_hang,     "vm-hang",     TYPE_ID_UINT64, MIN_VM_HANG, MAX_VM_HANG, NULL },
+	{ OPT_vm_keep,     "vm-keep",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_vm_madvise,  "vm-madvise",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_vm_madvise },
+	{ OPT_vm_method,   "vm-method",   TYPE_ID_SIZE_T_METHOD, 0, 0, stress_vm_method },
+	{ OPT_vm_locked,   "vm-locked",   TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_vm_populate, "vm-populate", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 stressor_info_t stress_vm_info = {
 	.stressor = stress_vm,
-	.set_default = stress_vm_set_default,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
