@@ -98,6 +98,8 @@ static int stress_xattr(stress_args_t *args)
 	if (hugevalue)
 		(void)shim_memset(hugevalue, 'X', hugevalue_sz - 1);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
